#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
"""remove replication controller

Revision ID: 859fb45df249
Revises: 1f196a3dabae
Create Date: 2016-08-09 13:46:24.052528

"""

# revision identifiers, used by Alembic.
revision = '859fb45df249'
down_revision = '1f196a3dabae'

from alembic import op  # noqa: E402


def upgrade():
    op.drop_table('replicationcontroller')



# Copyright (c) 2016 OpenStack Foundation
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_log import log as logging

from magnum.common import service
from magnum.tests import base


class TestMagnumService(base.BaseTestCase):

    @mock.patch.object(logging, 'register_options')
    @mock.patch.object(logging, 'setup')
    @mock.patch('magnum.common.config.set_config_defaults')
    @mock.patch('magnum.common.config.parse_args')
    def test_prepare_service_with_argv_not_none(self, mock_parse, mock_set,
                                                mock_setup, mock_reg):
        argv = 'foo'
        mock_parse.side_effect = lambda *args, **kwargs: None

        service.prepare_service(argv)

        mock_parse.assert_called_once_with(argv)
        mock_setup.assert_called_once_with(base.CONF, 'magnum')
        mock_reg.assert_called_once_with(base.CONF)
        mock_set.assert_called_once_with()

    @mock.patch.object(logging, 'register_options')
    @mock.patch.object(logging, 'setup')
    @mock.patch('magnum.common.config.set_config_defaults')
    @mock.patch('magnum.common.config.parse_args')
    def test_prepare_service_with_argv_none(self, mock_parse, mock_set,
                                            mock_setup, mock_reg):
        argv = None
        mock_parse.side_effect = lambda *args, **kwargs: None

        service.prepare_service(argv)

        mock_parse.assert_called_once_with([])
        mock_setup.assert_called_once_with(base.CONF, 'magnum')
        mock_reg.assert_called_once_with(base.CONF)
        mock_set.assert_called_once_with()



# Copyright 2016 Intel, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import mock

from magnum.common.cert_manager import x509keypair_cert_manager as x509_cm
from magnum.common import context
from magnum.tests import base
from magnum.tests.unit.db import base as db_base
from magnum.tests.unit.db import utils


class TestX509keypairCert(base.BaseTestCase):

    def setUp(self):
        self.certificate = "My Certificate"
        self.intermediates = "My Intermediates"
        self.private_key = "My Private Key"
        self.private_key_passphrase = "My Private Key Passphrase"

        super(TestX509keypairCert, self).setUp()

    def test_x509keypair_cert(self):
        # Create a cert
        cert = x509_cm.Cert(
            certificate=self.certificate,
            intermediates=self.intermediates,
            private_key=self.private_key,
            private_key_passphrase=self.private_key_passphrase
        )

        # Validate the cert functions
        self.assertEqual(self.certificate, cert.get_certificate())
        self.assertEqual(self.intermediates, cert.get_intermediates())
        self.assertEqual(self.private_key, cert.get_private_key())
        self.assertEqual(self.private_key_passphrase,
                         cert.get_private_key_passphrase())


class TestX509keypairManager(db_base.DbTestCase):

    def setUp(self):
        self.certificate = "My Certificate"
        self.intermediates = "My Intermediates"
        self.private_key = "My Private Key"
        self.private_key_passphrase = "My Private Key Passphrase"
        self.context = context.make_admin_context()
        super(TestX509keypairManager, self).setUp()

    def test_store_cert(self):
        x509keypair = utils.get_test_x509keypair()
        with mock.patch.object(self.dbapi, 'create_x509keypair',
                               autospec=True) as mock_create_x509keypair:
            mock_create_x509keypair.return_value = x509keypair

            uuid = x509_cm.CertManager.store_cert(context=self.context,
                                                  **x509keypair)
            self.assertEqual(uuid, '72625085-c507-4410-9b28-cd7cf1fbf1ad')

    def test_get_cert(self):
        x509keypair = utils.get_test_x509keypair(uuid='fake-uuid')
        with mock.patch.object(self.dbapi, 'get_x509keypair_by_uuid',
                               autospec=True) as mock_get_x509keypair:
            mock_get_x509keypair.return_value = x509keypair
            cert_obj = x509_cm.CertManager.get_cert('fake-uuid',
                                                    context=self.context)
            self.assertEqual(cert_obj.certificate, 'certificate')
            self.assertEqual(cert_obj.private_key, 'private_key')
            self.assertEqual(cert_obj.private_key_passphrase,
                             'private_key_passphrase')
            self.assertEqual(cert_obj.intermediates, 'intermediates')
            mock_get_x509keypair.assert_called_once_with(self.context,
                                                         'fake-uuid')

    def test_delete_cert(self):
        x509keypair = utils.get_test_x509keypair(uuid='fake-uuid')
        with mock.patch.object(self.dbapi, 'get_x509keypair_by_uuid',
                               autospec=True) as mock_get_x509keypair:
            mock_get_x509keypair.return_value = x509keypair
            with mock.patch.object(self.dbapi, 'destroy_x509keypair',
                                   autospec=True) as mock_destroy_x509keypair:
                x509_cm.CertManager.delete_cert('fake-uuid',
                                                context=self.context)
                mock_get_x509keypair.assert_called_once_with(self.context,
                                                             'fake-uuid')
                mock_destroy_x509keypair.assert_called_once_with('fake-uuid')



# Copyright 2015 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography import x509 as c_x509
from cryptography.x509.oid import NameOID
from unittest import mock

import six

from magnum.common import exception
from magnum.common.x509 import operations
from magnum.tests import base


class TestX509(base.BaseTestCase):

    def setUp(self):
        super(TestX509, self).setUp()
        self.issuer_name = six.u("fake-issuer")
        self.subject_name = six.u("fake-subject")
        self.organization_name = six.u("fake-organization")
        self.ca_encryption_password = six.b("fake-ca-password")
        self.encryption_password = six.b("fake-password")

    def _load_pems(self, keypairs, encryption_password):
        private_key = serialization.load_pem_private_key(
            keypairs['private_key'],
            password=encryption_password,
            backend=default_backend(),
        )
        certificate = c_x509.load_pem_x509_certificate(
            keypairs['certificate'], default_backend())

        return certificate, private_key

    def _generate_ca_certificate(self, issuer_name=None):
        issuer_name = issuer_name or self.issuer_name
        keypairs = operations.generate_ca_certificate(
            issuer_name, encryption_password=self.ca_encryption_password)

        return self._load_pems(keypairs, self.ca_encryption_password)

    def _generate_client_certificate(self, issuer_name, subject_name):
        ca = operations.generate_ca_certificate(
            self.issuer_name, encryption_password=self.ca_encryption_password)
        keypairs = operations.generate_client_certificate(
            self.issuer_name,
            self.subject_name,
            self.organization_name,
            ca['private_key'],
            encryption_password=self.encryption_password,
            ca_key_password=self.ca_encryption_password,
        )

        return self._load_pems(keypairs, self.encryption_password)

    def _public_bytes(self, public_key):
        return public_key.public_bytes(
            serialization.Encoding.PEM,
            serialization.PublicFormat.SubjectPublicKeyInfo
        )

    def _private_bytes(self, private_key):
        return private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        )

    def _generate_private_key(self):
        return rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048,
            backend=default_backend()
        )

    def _build_csr(self, private_key):
        csr = c_x509.CertificateSigningRequestBuilder()
        csr = csr.subject_name(c_x509.Name([
            c_x509.NameAttribute(NameOID.COMMON_NAME, self.subject_name)
        ]))

        return csr.sign(private_key, hashes.SHA256(), default_backend())

    def assertHasPublicKey(self, keypairs):
        key = keypairs[1]
        cert = keypairs[0]

        self.assertEqual(self._public_bytes(key.public_key()),
                         self._public_bytes(cert.public_key()))

    def assertHasSubjectName(self, cert, subject_name):
        actual_subject_name = cert.subject.get_attributes_for_oid(
            c_x509.NameOID.COMMON_NAME)
        actual_subject_name = actual_subject_name[0].value

        self.assertEqual(subject_name, actual_subject_name)

    def assertHasIssuerName(self, cert, issuer_name):
        actual_issuer_name = cert.issuer.get_attributes_for_oid(
            c_x509.NameOID.COMMON_NAME)
        actual_issuer_name = actual_issuer_name[0].value

        self.assertEqual(issuer_name, actual_issuer_name)

    def assertInClientExtensions(self, cert):
        key_usage = c_x509.KeyUsage(True, False, True, False, False, False,
                                    False, False, False)
        key_usage = c_x509.Extension(key_usage.oid, True, key_usage)
        extended_key_usage = c_x509.ExtendedKeyUsage([c_x509.OID_CLIENT_AUTH])
        extended_key_usage = c_x509.Extension(extended_key_usage.oid, False,
                                              extended_key_usage)
        basic_constraints = c_x509.BasicConstraints(ca=False, path_length=None)
        basic_constraints = c_x509.Extension(basic_constraints.oid, True,
                                             basic_constraints)

        self.assertIn(key_usage, cert.extensions)
        self.assertIn(extended_key_usage, cert.extensions)
        self.assertIn(basic_constraints, cert.extensions)

    def test_generate_ca_certificate_with_bytes_issuer_name(self):
        issuer_name = six.b("bytes-issuer-name")
        cert, _ = self._generate_ca_certificate(issuer_name)

        issuer_name = issuer_name.decode('utf-8')
        self.assertHasSubjectName(cert, issuer_name)
        self.assertHasIssuerName(cert, issuer_name)

    def test_generate_ca_certificate_has_publickey(self):
        keypairs = self._generate_ca_certificate(self.issuer_name)

        self.assertHasPublicKey(keypairs)

    def test_generate_ca_certificate_set_subject_name(self):
        cert, _ = self._generate_ca_certificate(self.issuer_name)

        self.assertHasSubjectName(cert, self.issuer_name)

    def test_generate_ca_certificate_set_issuer_name(self):
        cert, _ = self._generate_ca_certificate(self.issuer_name)

        self.assertHasIssuerName(cert, self.issuer_name)

    def test_generate_ca_certificate_set_extentions_as_ca(self):
        cert, _ = self._generate_ca_certificate(self.issuer_name)

        key_usage = c_x509.KeyUsage(False, False, False, False, False, True,
                                    False, False, False)
        key_usage = c_x509.Extension(key_usage.oid, True, key_usage)
        basic_constraints = c_x509.BasicConstraints(ca=True, path_length=0)
        basic_constraints = c_x509.Extension(basic_constraints.oid, True,
                                             basic_constraints)

        self.assertIn(key_usage, cert.extensions)
        self.assertIn(basic_constraints, cert.extensions)

    def test_generate_client_certificate_has_publickey(self):
        keypairs = self._generate_client_certificate(
            self.issuer_name, self.subject_name)

        self.assertHasPublicKey(keypairs)

    def test_generate_client_certificate_set_subject_name(self):
        cert, _ = self._generate_client_certificate(
            self.issuer_name, self.subject_name)

        self.assertHasSubjectName(cert, self.subject_name)

    def test_generate_client_certificate_set_issuer_name(self):
        cert, key = self._generate_client_certificate(
            self.issuer_name, self.subject_name)

        self.assertHasIssuerName(cert, self.issuer_name)

    def test_generate_client_certificate_set_extentions_as_client(self):
        cert, key = self._generate_client_certificate(
            self.issuer_name, self.subject_name)

        self.assertInClientExtensions(cert)

    def test_load_pem_private_key_with_bytes_private_key(self):
        private_key = self._generate_private_key()
        private_key = self._private_bytes(private_key)

        self.assertIsInstance(private_key, six.binary_type)
        private_key = operations._load_pem_private_key(private_key)
        self.assertIsInstance(private_key, rsa.RSAPrivateKey)

    def test_load_pem_private_key_with_unicode_private_key(self):
        private_key = self._generate_private_key()
        private_key = self._private_bytes(private_key)
        private_key = six.text_type(private_key.decode('utf-8'))

        self.assertIsInstance(private_key, six.text_type)
        private_key = operations._load_pem_private_key(private_key)
        self.assertIsInstance(private_key, rsa.RSAPrivateKey)

    @mock.patch('cryptography.x509.load_pem_x509_csr')
    @mock.patch('six.b')
    def test_sign_with_unicode_csr(self, mock_six, mock_load_pem):
        ca_key = self._generate_private_key()
        private_key = self._generate_private_key()
        csr_obj = self._build_csr(private_key)
        csr = csr_obj.public_bytes(serialization.Encoding.PEM)
        csr = six.text_type(csr.decode('utf-8'))

        mock_load_pem.return_value = csr_obj
        operations.sign(csr, self.issuer_name, ca_key,
                        skip_validation=True)
        mock_six.assert_called_once_with(csr)

    @mock.patch('cryptography.x509.load_pem_x509_csr')
    def test_sign_empty_chars(self, mock_load_pem):
        ca_key = self._generate_private_key()
        private_key = self._generate_private_key()
        csr_obj = self._build_csr(private_key)
        csr = csr_obj.public_bytes(serialization.Encoding.PEM)
        csr = six.text_type(csr.decode('utf-8'))

        mock_load_pem.return_value = csr_obj
        certificate = operations.sign(csr, self.issuer_name,
                                      ca_key, skip_validation=True)

        # Certificate has to be striped for some parsers
        self.assertEqual(certificate,
                         certificate.strip())

    def test_sign_with_invalid_csr(self):
        ca_key = self._generate_private_key()
        csr = 'test'
        csr = six.u(csr)

        self.assertRaises(exception.InvalidCsr,
                          operations.sign,
                          csr, self.issuer_name, ca_key, skip_validation=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import itertools

from oslo_config import cfg

from magnum.i18n import _

neutron_group = cfg.OptGroup(name='neutron_client',
                             title='Options for the neutron client')

neutron_client_opts = [
    cfg.StrOpt('region_name',
               help=_('Region in Identity service catalog to use for '
                      'communication with the OpenStack service.')),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               help=_('Type of endpoint in Identity service catalog to use '
                      'for communication with the OpenStack service.'))]

common_security_opts = [
    cfg.StrOpt('ca_file',
               help=_('Optional CA cert file to use in SSL connections.')),
    cfg.StrOpt('cert_file',
               help=_('Optional PEM-formatted certificate chain file.')),
    cfg.StrOpt('key_file',
               help=_('Optional PEM-formatted file that contains the '
                      'private key.')),
    cfg.BoolOpt('insecure',
                default=False,
                help=_("If set, then the server's certificate will not "
                       "be verified."))]

ALL_OPTS = list(itertools.chain(
    neutron_client_opts,
    common_security_opts
))


def register_opts(conf):
    conf.register_group(neutron_group)
    conf.register_opts(ALL_OPTS, group=neutron_group)


def list_opts():
    return {
        neutron_group: ALL_OPTS
    }



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from alembic import op
import sqlalchemy as sa

"""add-master_lb_enabled-to-cluster

Revision ID: 95096e2334ee
Revises: c04e925e65c2
Create Date: 2020-06-26 14:33:05.529200

"""

# revision identifiers, used by Alembic.
revision = '95096e2334ee'
down_revision = 'c04e925e65c2'


def upgrade():
    op.add_column('cluster',
                  sa.Column('master_lb_enabled',
                            sa.Boolean(create_constraint=False),
                            default=False))
    # Populate existing cluster with the cluster template_id
    connection = op.get_bind()
    connection.execute(
        "UPDATE cluster "
        "INNER JOIN cluster_template "
        "ON cluster_template.uuid=cluster.cluster_template_id "
        "SET cluster.master_lb_enabled=cluster_template.master_lb_enabled "
        "WHERE cluster_template.uuid=cluster.cluster_template_id and "
        "cluster.master_lb_enabled is NULL"
    )



# Copyright 2016 Yahoo! Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Tests for manipulating Quota via the DB API"""

from magnum.common import exception
from magnum.tests.unit.db import base
from magnum.tests.unit.db import utils


class DbQuotaTestCase(base.DbTestCase):

    def test_create_quota(self):
        utils.create_test_quotas()

    def test_create_quota_already_exists(self):
        utils.create_test_quotas()
        self.assertRaises(exception.QuotaAlreadyExists,
                          utils.create_test_quotas)

    def test_get_quota_all(self):
        q = utils.create_test_quotas()
        res = self.dbapi.quota_get_all_by_project_id(
            project_id='fake_project')
        for r in res:
            self.assertEqual(q.id, r.id)
            self.assertEqual(q.hard_limit, r.hard_limit)
            self.assertEqual(q.project_id, r.project_id)
            self.assertEqual(q.resource, r.resource)

    def test_get_quota_by_project_id_resource(self):
        q = utils.create_test_quotas(project_id='123',
                                     resource='test-res',
                                     hard_limit=5)
        res = self.dbapi.get_quota_by_project_id_resource('123', 'test-res')
        self.assertEqual(q.hard_limit, res.hard_limit)
        self.assertEqual(q.project_id, res.project_id)
        self.assertEqual(q.resource, res.resource)

    def test_get_quota_by_project_id_resource_not_found(self):
        utils.create_test_quotas(project_id='123',
                                 resource='test-res',
                                 hard_limit=5)
        self.assertRaises(exception.QuotaNotFound,
                          self.dbapi.get_quota_by_project_id_resource,
                          project_id='123',
                          resource='bad-res')

    def test_get_quota_list(self):
        project_ids = []
        for i in range(1, 6):
            project_id = 'proj-'+str(i)
            utils.create_test_quotas(project_id=project_id)
            project_ids.append(project_id)
        res = self.dbapi.get_quota_list(self.context)
        res_proj_ids = [r.project_id for r in res]
        self.assertEqual(sorted(project_ids), sorted(res_proj_ids))

    def test_get_quota_list_sorted(self):
        project_ids = []
        for i in range(1, 6):
            project_id = 'proj-'+str(i)
            utils.create_test_quotas(project_id=project_id)
            project_ids.append(project_id)
        res = self.dbapi.get_quota_list(self.context, sort_key='project_id')
        res_proj_ids = [r.project_id for r in res]
        self.assertEqual(sorted(project_ids), res_proj_ids)

    def test_get_quota_list_invalid_sort_key(self):
        project_ids = []
        for i in range(1, 6):
            project_id = 'proj-'+str(i)
            utils.create_test_quotas(project_id=project_id)
            project_ids.append(project_id)

        self.assertRaises(exception.InvalidParameterValue,
                          self.dbapi.get_quota_list,
                          self.context,
                          sort_key='invalid')

    def test_get_quota_list_with_filters(self):
        quota1 = utils.create_test_quotas(project_id='proj-1', resource='res1')
        quota2 = utils.create_test_quotas(project_id='proj-1', resource='res2')
        quota3 = utils.create_test_quotas(project_id='proj-2', resource='res1')

        res = self.dbapi.get_quota_list(
            self.context, filters={'resource': 'res2'})
        self.assertEqual(quota2.project_id, res[0].project_id)

        res = self.dbapi.get_quota_list(
            self.context, filters={'project_id': 'proj-2'})
        self.assertEqual(quota3.project_id, res[0].project_id)

        res = self.dbapi.get_quota_list(
            self.context, filters={'project_id': 'proj-1'})
        self.assertEqual(sorted([quota1.project_id, quota2.project_id]),
                         sorted([r.project_id for r in res]))

    def test_update_quota(self):
        q = utils.create_test_quotas(hard_limit=5,
                                     project_id='1234',
                                     resource='Cluster')

        res = self.dbapi.get_quota_by_project_id_resource('1234', 'Cluster')
        self.assertEqual(q.hard_limit, res.hard_limit)
        self.assertEqual(q.project_id, res.project_id)
        self.assertEqual(q.resource, res.resource)
        quota_dict = {'resource': 'Cluster', 'hard_limit': 15}
        self.dbapi.update_quota('1234', quota_dict)
        res = self.dbapi.get_quota_by_project_id_resource('1234', 'Cluster')
        self.assertEqual(quota_dict['hard_limit'], res.hard_limit)
        self.assertEqual(quota_dict['resource'], res.resource)

    def test_update_quota_not_found(self):
        utils.create_test_quotas(hard_limit=5,
                                 project_id='1234',
                                 resource='Cluster')
        quota_dict = {'resource': 'Cluster', 'hard_limit': 15}
        self.assertRaises(exception.QuotaNotFound,
                          self.dbapi.update_quota,
                          'invalid_proj',
                          quota_dict)

    def test_delete_quota(self):
        q = utils.create_test_quotas(project_id='123',
                                     resource='test-res',
                                     hard_limit=5)
        res = self.dbapi.get_quota_by_project_id_resource('123', 'test-res')
        self.assertEqual(q.hard_limit, res.hard_limit)
        self.assertEqual(q.project_id, res.project_id)
        self.assertEqual(q.resource, res.resource)
        self.dbapi.delete_quota(q.project_id, q.resource)
        self.assertRaises(exception.QuotaNotFound,
                          self.dbapi.get_quota_by_project_id_resource,
                          project_id='123',
                          resource='bad-res')

    def test_delete_quota_that_does_not_exist(self):
        # Make sure that quota does not exist
        self.assertRaises(exception.QuotaNotFound,
                          self.dbapi.get_quota_by_project_id_resource,
                          project_id='123',
                          resource='bad-res')

        # Now try to delete non-existing quota
        self.assertRaises(exception.QuotaNotFound,
                          self.dbapi.delete_quota,
                          project_id='123',
                          resource='bad-res')



# Copyright (c) 2015 Intel Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools

from oslo_log import log
from oslo_log.versionutils import deprecated
from oslo_service import loopingcall
from oslo_service import periodic_task

from pycadf import cadftaxonomy as taxonomy

from magnum.common import context
from magnum.common import profiler
from magnum.common import rpc
from magnum.conductor import monitors
from magnum.conductor import utils as conductor_utils
import magnum.conf
from magnum.drivers.common import driver
from magnum import objects


CONF = magnum.conf.CONF
LOG = log.getLogger(__name__)


def set_context(func):
    @functools.wraps(func)
    def handler(self, ctx):
        ctx = context.make_admin_context(all_tenants=True)
        context.set_ctx(ctx)
        func(self, ctx)
        context.set_ctx(None)
    return handler


class ClusterUpdateJob(object):

    status_to_event = {
        objects.fields.ClusterStatus.DELETE_COMPLETE: taxonomy.ACTION_DELETE,
        objects.fields.ClusterStatus.CREATE_COMPLETE: taxonomy.ACTION_CREATE,
        objects.fields.ClusterStatus.UPDATE_COMPLETE: taxonomy.ACTION_UPDATE,
        objects.fields.ClusterStatus.ROLLBACK_COMPLETE: taxonomy.ACTION_UPDATE,
        objects.fields.ClusterStatus.CREATE_FAILED: taxonomy.ACTION_CREATE,
        objects.fields.ClusterStatus.DELETE_FAILED: taxonomy.ACTION_DELETE,
        objects.fields.ClusterStatus.UPDATE_FAILED: taxonomy.ACTION_UPDATE,
        objects.fields.ClusterStatus.ROLLBACK_FAILED: taxonomy.ACTION_UPDATE
    }

    def __init__(self, ctx, cluster):
        self.ctx = ctx
        self.cluster = cluster

    def update_status(self):
        LOG.debug("Updating status for cluster %s", self.cluster.id)
        # get the driver for the cluster
        cdriver = driver.Driver.get_driver_for_cluster(self.ctx, self.cluster)
        # ask the driver to sync status
        cdriver.update_cluster_status(self.ctx, self.cluster)
        LOG.debug("Status for cluster %s updated to %s (%s)",
                  self.cluster.id, self.cluster.status,
                  self.cluster.status_reason)
        # status update notifications
        if self.cluster.status.endswith("_COMPLETE"):
            conductor_utils.notify_about_cluster_operation(
                self.ctx, self.status_to_event[self.cluster.status],
                taxonomy.OUTCOME_SUCCESS, self.cluster)
        if self.cluster.status.endswith("_FAILED"):
            conductor_utils.notify_about_cluster_operation(
                self.ctx, self.status_to_event[self.cluster.status],
                taxonomy.OUTCOME_FAILURE, self.cluster)
        # if we're done with it, delete it
        if self.cluster.status == objects.fields.ClusterStatus.DELETE_COMPLETE:
            # delete all the nodegroups that belong to this cluster
            for ng in objects.NodeGroup.list(self.ctx, self.cluster.uuid):
                ng.destroy()
            self.cluster.destroy()
        # end the "loop"
        raise loopingcall.LoopingCallDone()


class ClusterHealthUpdateJob(object):

    def __init__(self, ctx, cluster):
        self.ctx = ctx
        self.cluster = cluster

    def _update_health_status(self):
        monitor = monitors.create_monitor(self.ctx, self.cluster)
        if monitor is None:
            return

        try:
            monitor.poll_health_status()
        except Exception as e:
            LOG.warning(
                "Skip pulling data from cluster %(cluster)s due to "
                "error: %(e)s",
                {'e': e, 'cluster': self.cluster.uuid}, exc_info=True)
            # TODO(flwang): Should we mark this cluster's health status as
            # UNKNOWN if Magnum failed to pull data from the cluster? Because
            # that basically means the k8s API doesn't work at that moment.
            return

        if monitor.data.get('health_status'):
            self.cluster.health_status = monitor.data.get('health_status')
            self.cluster.health_status_reason = monitor.data.get(
                'health_status_reason')
            self.cluster.save()

    def update_health_status(self):
        LOG.debug("Updating health status for cluster %s", self.cluster.id)
        self._update_health_status()
        LOG.debug("Status for cluster %s updated to %s (%s)",
                  self.cluster.id, self.cluster.health_status,
                  self.cluster.health_status_reason)
        # TODO(flwang): Health status update notifications?
        # end the "loop"
        raise loopingcall.LoopingCallDone()


@profiler.trace_cls("rpc")
class MagnumPeriodicTasks(periodic_task.PeriodicTasks):
    """Magnum periodic Task class

    Any periodic task job need to be added into this class

    NOTE(suro-patz):
    - oslo_service.periodic_task runs tasks protected within try/catch
      block, with default raise_on_error as 'False', in run_periodic_tasks(),
      which ensures the process does not die, even if a task encounters an
      Exception.
    - The periodic tasks here does not necessarily need another
      try/catch block. The present try/catch block here helps putting
      magnum-periodic-task-specific log/error message.

    """

    def __init__(self, conf):
        super(MagnumPeriodicTasks, self).__init__(conf)
        self.notifier = rpc.get_notifier()

    @periodic_task.periodic_task(
        spacing=CONF.kubernetes.health_polling_interval,
        run_immediately=True)
    @set_context
    def sync_cluster_status(self, ctx):
        try:
            LOG.debug('Starting to sync up cluster status')

            # get all the clusters that are IN_PROGRESS
            status = [objects.fields.ClusterStatus.CREATE_IN_PROGRESS,
                      objects.fields.ClusterStatus.UPDATE_IN_PROGRESS,
                      objects.fields.ClusterStatus.DELETE_IN_PROGRESS,
                      objects.fields.ClusterStatus.ROLLBACK_IN_PROGRESS]
            filters = {'status': status}
            clusters = objects.Cluster.list(ctx, filters=filters)
            if not clusters:
                return

            # synchronize with underlying orchestration
            for cluster in clusters:
                job = ClusterUpdateJob(ctx, cluster)
                # though this call isn't really looping, we use this
                # abstraction anyway to avoid dealing directly with eventlet
                # hooey
                lc = loopingcall.FixedIntervalLoopingCall(f=job.update_status)
                lc.start(1, stop_on_exception=True)

        except Exception as e:
            LOG.warning(
                "Ignore error [%s] when syncing up cluster status.",
                e, exc_info=True)

    @periodic_task.periodic_task(spacing=10, run_immediately=True)
    @set_context
    def sync_cluster_health_status(self, ctx):
        try:
            LOG.debug('Starting to sync up cluster health status')

            status = [objects.fields.ClusterStatus.CREATE_COMPLETE,
                      objects.fields.ClusterStatus.UPDATE_COMPLETE,
                      objects.fields.ClusterStatus.UPDATE_IN_PROGRESS,
                      objects.fields.ClusterStatus.ROLLBACK_IN_PROGRESS]
            filters = {'status': status}
            clusters = objects.Cluster.list(ctx, filters=filters)
            if not clusters:
                return

            # synchronize using native COE API
            for cluster in clusters:
                job = ClusterHealthUpdateJob(ctx, cluster)
                # though this call isn't really looping, we use this
                # abstraction anyway to avoid dealing directly with eventlet
                # hooey
                lc = loopingcall.FixedIntervalLoopingCall(
                    f=job.update_health_status)
                lc.start(1, stop_on_exception=True)

        except Exception as e:
            LOG.warning(
                "Ignore error [%s] when syncing up cluster status.",
                e, exc_info=True)

    @periodic_task.periodic_task(run_immediately=True)
    @set_context
    @deprecated(as_of=deprecated.ROCKY)
    def _send_cluster_metrics(self, ctx):
        if not CONF.drivers.send_cluster_metrics:
            LOG.debug('Skip sending cluster metrics')
            return

        LOG.debug('Starting to send cluster metrics')
        for cluster in objects.Cluster.list(ctx):
            if cluster.status not in (
                    objects.fields.ClusterStatus.CREATE_COMPLETE,
                    objects.fields.ClusterStatus.UPDATE_COMPLETE):
                continue

            monitor = monitors.create_monitor(ctx, cluster)
            if monitor is None:
                continue

            try:
                monitor.pull_data()
            except Exception as e:
                LOG.warning(
                    "Skip pulling data from cluster %(cluster)s due to "
                    "error: %(e)s",
                    {'e': e, 'cluster': cluster.uuid}, exc_info=True)
                continue

            metrics = list()
            for name in monitor.get_metric_names():
                try:
                    metric = {
                        'name': name,
                        'value': monitor.compute_metric_value(name),
                        'unit': monitor.get_metric_unit(name),
                    }
                    metrics.append(metric)
                except Exception as e:
                    LOG.warning("Skip adding metric %(name)s due to "
                                "error: %(e)s",
                                {'e': e, 'name': name}, exc_info=True)

            message = dict(metrics=metrics,
                           user_id=cluster.user_id,
                           project_id=cluster.project_id,
                           resource_id=cluster.uuid)
            LOG.debug("About to send notification: '%s'", message)
            self.notifier.info(ctx, "magnum.cluster.metrics.update",
                               message)


def setup(conf, tg):
    pt = MagnumPeriodicTasks(conf)
    tg.add_dynamic_timer(
        pt.run_periodic_tasks,
        periodic_interval_max=conf.periodic_interval_max,
        context=None)



# Copyright 2014 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class VersionedMethod(object):

    def __init__(self, name, start_version, end_version, func):
        """Versioning information for a single method

        @name: Name of the method
        @start_version: Minimum acceptable version
        @end_version: Maximum acceptable version
        @func: Method to call

        Minimum and maximum are inclusive
        """
        self.name = name
        self.start_version = start_version
        self.end_version = end_version
        self.func = func

    def __str__(self):
        return ("Version Method %s: min: %s, max: %s"
                % (self.name, self.start_version, self.end_version))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest

from cryptography import x509 as c_x509

from magnum.common.exception import CertificateValidationError
from magnum.common.x509 import validator as v


class TestValidators(unittest.TestCase):
    def _build_key_usage(self, critical=False):
        # Digital Signature and Key Encipherment are enabled
        key_usage = c_x509.KeyUsage(
            True, False, True, False, False, False, False, False, False)
        return c_x509.Extension(key_usage.oid, critical, key_usage)

    def _build_basic_constraints(self, ca=False, critical=False):
        bc = c_x509.BasicConstraints(ca, None)
        return c_x509.Extension(bc.oid, critical, bc)

    def test_filter_allowed_extensions(self):
        key_usage = self._build_key_usage(critical=True)

        actual = [e for e in v.filter_allowed_extensions([key_usage],
                                                         ['keyUsage'])]
        self.assertEqual([key_usage], actual)

    def test_filter_allowed_extensions_disallowed_but_not_critical(self):
        key_usage = self._build_key_usage()

        actual = [e for e in v.filter_allowed_extensions([key_usage],
                                                         ['subjectAltName'])]

        self.assertEqual([], actual)

    def test_filter_allowed_extensions_disallowed(self):
        key_usage = self._build_key_usage(critical=True)

        with self.assertRaises(CertificateValidationError):
            next(v.filter_allowed_extensions([key_usage], ['subjectAltName']))

    def test_merge_key_usage(self):
        key_usage = self._build_key_usage(critical=True)

        self.assertEqual(key_usage,
                         v._merge_key_usage(key_usage,
                                            ['Digital Signature',
                                             'Key Encipherment']))

    def test_merge_key_usage_disallowed_but_not_critical(self):
        key_usage = self._build_key_usage()
        expected = c_x509.KeyUsage(
            True, False, False, False, False, False, False, False, False)
        expected = c_x509.Extension(expected.oid, False, expected)

        self.assertEqual(expected,
                         v._merge_key_usage(key_usage,
                                            ['Digital Signature']))

    def test_merge_key_usage_disallowed(self):
        key_usage = self._build_key_usage(critical=True)

        with self.assertRaises(CertificateValidationError):
            v._merge_key_usage(key_usage, ['Digital Signature'])

    def test_disallow_ca_in_basic_constraints_not_critical(self):
        bc = self._build_basic_constraints(ca=True)
        expected = self._build_basic_constraints(ca=False)

        self.assertEqual(expected, v._disallow_ca_in_basic_constraints(bc))

    def test_disallow_ca_in_basic_constraints(self):
        bc = self._build_basic_constraints(ca=True, critical=True)

        with self.assertRaises(CertificateValidationError):
            v._disallow_ca_in_basic_constraints(bc)

    def test_disallow_ca_in_basic_constraints_with_non_ca(self):
        bc = self._build_basic_constraints(ca=False)

        self.assertEqual(bc, v._disallow_ca_in_basic_constraints(bc))

    def test_remove_ca_key_usage(self):
        contains_ca_key_usage = set([
            "Digital Signature", "Certificate Sign", "CRL Sign"])

        self.assertEqual(set(["Digital Signature"]),
                         v._remove_ca_key_usage(contains_ca_key_usage))

    def test_remove_ca_key_usage_cert_sign(self):
        contains_ca_key_usage = set(["Digital Signature", "Certificate Sign"])

        self.assertEqual(set(["Digital Signature"]),
                         v._remove_ca_key_usage(contains_ca_key_usage))

    def test_remove_ca_key_usage_crl_sign(self):
        contains_ca_key_usage = set(["Digital Signature", "CRL Sign"])

        self.assertEqual(set(["Digital Signature"]),
                         v._remove_ca_key_usage(contains_ca_key_usage))

    def test_remove_ca_key_usage_without_ca_usage(self):
        contains_ca_key_usage = set(["Digital Signature"])

        self.assertEqual(set(["Digital Signature"]),
                         v._remove_ca_key_usage(contains_ca_key_usage))



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneauth1 import loading as ka_loading
from oslo_config import cfg

CFG_GROUP = 'keystone_auth'
CFG_LEGACY_GROUP = 'keystone_authtoken'

legacy_session_opts = {
    'certfile': [cfg.DeprecatedOpt('certfile', CFG_LEGACY_GROUP)],
    'keyfile': [cfg.DeprecatedOpt('keyfile', CFG_LEGACY_GROUP)],
    'cafile': [cfg.DeprecatedOpt('cafile', CFG_LEGACY_GROUP)],
    'insecure': [cfg.DeprecatedOpt('insecure', CFG_LEGACY_GROUP)],
    'timeout': [cfg.DeprecatedOpt('timeout', CFG_LEGACY_GROUP)],
}

keystone_auth_group = cfg.OptGroup(name=CFG_GROUP,
                                   title='Options for Keystone in Magnum')


def register_opts(conf):
    # FIXME(pauloewerton): remove import of authtoken group and legacy options
    # after deprecation period
    conf.import_group(CFG_LEGACY_GROUP, 'keystonemiddleware.auth_token')
    ka_loading.register_auth_conf_options(conf, CFG_GROUP)
    ka_loading.register_session_conf_options(
        conf, CFG_GROUP, deprecated_opts=legacy_session_opts)
    conf.set_default('auth_type', default='password', group=CFG_GROUP)


def list_opts():
    keystone_auth_opts = (ka_loading.get_auth_common_conf_options() +
                          ka_loading.get_auth_plugin_conf_options('password'))
    return {
        keystone_auth_group: keystone_auth_opts
    }



# Copyright 2015 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from magnum.conductor import tasks


class CreateStack(tasks.OSBaseTask):
    """CreateStack Task

    This task interfaces with Heat API and creates a stack based on parameters
    provided to the Task.

    """

    def execute(self, stack_name, parameters, template, files):
        stack = self.os_client.stacks.create(stack_name=stack_name,
                                             parameters=parameters,
                                             template=template, files=files)
        return stack


class UpdateStack(tasks.OSBaseTask):
    """UpdateStack Task

    This task interfaces with Heat API and update a stack based on parameters
    provided to the Task.

    """

    def execute(self, stack_id, parameters, template, files):
        self.os_client.stacks.update(stack_id, parameters=parameters,
                                     template=template, files=files)


class DeleteStack(tasks.OSBaseTask):
    """DeleteStack Task

    This task interfaces with Heat API and delete a stack based on parameters
    provided to the Task.

    """

    def execute(self, stack_id):
        self.os_client.stacks.delete(stack_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import random
import socket
import string
import struct

from tempest.lib.common.utils import data_utils

from magnum.tests.functional.api.v1.models import bay_model
from magnum.tests.functional.api.v1.models import baymodel_model
from magnum.tests.functional.api.v1.models import baymodelpatch_model
from magnum.tests.functional.api.v1.models import baypatch_model
from magnum.tests.functional.api.v1.models import cert_model
from magnum.tests.functional.api.v1.models import cluster_model
from magnum.tests.functional.api.v1.models import cluster_template_model
from magnum.tests.functional.api.v1.models import cluster_templatepatch_model
from magnum.tests.functional.api.v1.models import clusterpatch_model
from magnum.tests.functional.common import config


def random_int(min_int=1, max_int=100):
    return random.randrange(min_int, max_int)


def gen_coe_dep_network_driver(coe):
    allowed_driver_types = {
        'kubernetes': ['flannel', None],
        'swarm': ['docker', 'flannel', None],
        'swarm-mode': ['docker', None],
        'mesos': ['docker', None],
    }
    driver_types = allowed_driver_types[coe]
    return driver_types[random.randrange(0, len(driver_types))]


def gen_coe_dep_volume_driver(coe):
    allowed_driver_types = {
        'kubernetes': ['cinder', None],
        'swarm': ['rexray', None],
        'swarm-mode': ['rexray', None],
        'mesos': ['rexray', None],
    }
    driver_types = allowed_driver_types[coe]
    return driver_types[random.randrange(0, len(driver_types))]


def gen_random_port():
    return random_int(49152, 65535)


def gen_docker_volume_size(min_int=3, max_int=5):
    return random_int(min_int, max_int)


def gen_fake_ssh_pubkey():
    chars = "".join(
        random.choice(string.ascii_uppercase +
                      string.ascii_letters + string.digits + '/+=')
        for _ in range(372))
    return "ssh-rsa " + chars


def gen_random_ip():
    return socket.inet_ntoa(struct.pack('>I', random.randint(1, 0xffffffff)))


def gen_url(scheme="http", domain="example.com", port=80):
    return "%s://%s:%s" % (scheme, domain, port)


def gen_http_proxy():
    return gen_url(port=gen_random_port())


def gen_https_proxy():
    return gen_url(scheme="https", port=gen_random_port())


def gen_no_proxy():
    return ",".join(gen_random_ip() for x in range(3))


def baymodel_data(**kwargs):
    """Generates random baymodel data

    Keypair and image id cannot be random for the baymodel to be valid due to
    validations for the presence of keypair and image id prior to baymodel
    creation.

    :param keypair_id: keypair name
    :param image_id: image id or name
    :returns: BayModelEntity with generated data
    """

    data = {
        "name": data_utils.rand_name('bay'),
        "coe": "swarm-mode",
        "tls_disabled": False,
        "network_driver": None,
        "volume_driver": None,
        "labels": {},
        "public": False,
        "dns_nameserver": "8.8.8.8",
        "flavor_id": data_utils.rand_name('bay'),
        "master_flavor_id": data_utils.rand_name('bay'),
        "external_network_id": config.Config.nic_id,
        "keypair_id": data_utils.rand_name('bay'),
        "image_id": data_utils.rand_name('bay')
    }

    data.update(kwargs)
    model = baymodel_model.BayModelEntity.from_dict(data)

    return model


def baymodel_replace_patch_data(path, value=data_utils.rand_name('bay')):
    """Generates random baymodel patch data

    :param path: path to replace
    :param value: value to replace in patch
    :returns: BayModelPatchCollection with generated data
    """

    data = [{
        "path": path,
        "value": value,
        "op": "replace"
    }]
    return baymodelpatch_model.BayModelPatchCollection.from_dict(data)


def baymodel_remove_patch_data(path):
    """Generates baymodel patch data by removing value

    :param path: path to remove
    :returns: BayModelPatchCollection with generated data
    """

    data = [{
        "path": path,
        "op": "remove"
    }]
    return baymodelpatch_model.BayModelPatchCollection.from_dict(data)


def baymodel_data_with_valid_keypair_image_flavor():
    """Generates random baymodel data with valid keypair,image and flavor

    :returns: BayModelEntity with generated data
    """

    return baymodel_data(keypair_id=config.Config.keypair_id,
                         image_id=config.Config.image_id,
                         flavor_id=config.Config.flavor_id,
                         master_flavor_id=config.Config.master_flavor_id)


def baymodel_data_with_missing_image():
    """Generates random baymodel data with missing image

    :returns: BayModelEntity with generated data
    """

    return baymodel_data(keypair_id=config.Config.keypair_id,
                         flavor_id=config.Config.flavor_id,
                         master_flavor_id=config.Config.master_flavor_id)


def baymodel_data_with_missing_flavor():
    """Generates random baymodel data with missing flavor

    :returns: BayModelEntity with generated data
    """

    return baymodel_data(keypair_id=config.Config.keypair_id,
                         image_id=config.Config.image_id)


def baymodel_data_with_missing_keypair():
    """Generates random baymodel data with missing keypair

    :returns: BayModelEntity with generated data
    """

    return baymodel_data(image_id=config.Config.image_id,
                         flavor_id=config.Config.flavor_id,
                         master_flavor_id=config.Config.master_flavor_id)


def baymodel_valid_data_with_specific_coe(coe):
    """Generates random baymodel data with valid keypair and image

    :param coe: coe
    :returns: BayModelEntity with generated data
    """

    return baymodel_data(keypair_id=config.Config.keypair_id,
                         image_id=config.Config.image_id, coe=coe)


def valid_swarm_mode_baymodel(is_public=False):
    """Generates a valid swarm mode baymodel with valid data

    :returns: BayModelEntity with generated data
    """

    return baymodel_data(image_id=config.Config.image_id,
                         flavor_id=config.Config.flavor_id, public=is_public,
                         dns_nameserver=config.Config.dns_nameserver,
                         master_flavor_id=config.Config.master_flavor_id,
                         keypair_id=config.Config.keypair_id, coe="swarm-mode",
                         cluster_distro=None,
                         external_network_id=config.Config.nic_id,
                         http_proxy=None, https_proxy=None, no_proxy=None,
                         network_driver=None, volume_driver=None, labels={},
                         tls_disabled=False)


def bay_data(name=data_utils.rand_name('bay'),
             baymodel_id=data_utils.rand_uuid(),
             node_count=random_int(1, 5), discovery_url=gen_random_ip(),
             bay_create_timeout=random_int(1, 30),
             master_count=random_int(1, 5)):
    """Generates random bay data

    BayModel_id cannot be random for the bay to be valid due to
    validations for the presence of baymodel prior to baymodel
    creation.

    :param name: bay name (must be unique)
    :param baymodel_id: baymodel unique id (must already exist)
    :param node_count: number of agents for bay
    :param discovery_url: url provided for node discovery
    :param bay_create_timeout: timeout in minutes for bay create
    :param master_count: number of master nodes for the bay
    :returns: BayEntity with generated data
    """

    data = {
        "name": name,
        "baymodel_id": baymodel_id,
        "node_count": node_count,
        "discovery_url": None,
        "bay_create_timeout": bay_create_timeout,
        "master_count": master_count
    }
    model = bay_model.BayEntity.from_dict(data)

    return model


def valid_bay_data(baymodel_id, name=data_utils.rand_name('bay'), node_count=1,
                   master_count=1, bay_create_timeout=None):
    """Generates random bay data with valid

    :param baymodel_id: baymodel unique id that already exists
    :param name: bay name (must be unique)
    :param node_count: number of agents for bay
    :returns: BayEntity with generated data
    """

    return bay_data(baymodel_id=baymodel_id, name=name,
                    master_count=master_count, node_count=node_count,
                    bay_create_timeout=bay_create_timeout)


def bay_name_patch_data(name=data_utils.rand_name('bay')):
    """Generates random baymodel patch data

    :param name: name to replace in patch
    :returns: BayPatchCollection with generated data
    """

    data = [{
        "path": "/name",
        "value": name,
        "op": "replace"
    }]
    return baypatch_model.BayPatchCollection.from_dict(data)


def bay_api_addy_patch_data(address='0.0.0.0'):
    """Generates random bay patch data

    :param name: name to replace in patch
    :returns: BayPatchCollection with generated data
    """

    data = [{
        "path": "/api_address",
        "value": address,
        "op": "replace"
    }]
    return baypatch_model.BayPatchCollection.from_dict(data)


def bay_node_count_patch_data(node_count=2):
    """Generates random bay patch data

    :param name: name to replace in patch
    :returns: BayPatchCollection with generated data
    """

    data = [{
        "path": "/node_count",
        "value": node_count,
        "op": "replace"
    }]
    return baypatch_model.BayPatchCollection.from_dict(data)


def cert_data(cluster_uuid, csr_data):
    data = {
        "cluster_uuid": cluster_uuid,
        "csr": csr_data}

    model = cert_model.CertEntity.from_dict(data)

    return model


def cluster_template_data(**kwargs):
    """Generates random cluster_template data

    Keypair and image id cannot be random for the cluster_template to be valid
    due to validations for the presence of keypair and image id prior to
    cluster_template creation.

    :param keypair_id: keypair name
    :param image_id: image id or name
    :returns: ClusterTemplateEntity with generated data
    """

    data = {
        "name": data_utils.rand_name('cluster'),
        "coe": "swarm-mode",
        "tls_disabled": False,
        "network_driver": None,
        "volume_driver": None,
        "labels": {},
        "public": False,
        "dns_nameserver": "8.8.8.8",
        "flavor_id": data_utils.rand_name('cluster'),
        "master_flavor_id": data_utils.rand_name('cluster'),
        "external_network_id": config.Config.nic_id,
        "keypair_id": data_utils.rand_name('cluster'),
        "image_id": data_utils.rand_name('cluster')
    }

    data.update(kwargs)
    model = cluster_template_model.ClusterTemplateEntity.from_dict(data)

    return model


def cluster_template_replace_patch_data(path,
                                        value=data_utils.rand_name('cluster')):
    """Generates random ClusterTemplate patch data

    :param path: path to replace
    :param value: value to replace in patch
    :returns: ClusterTemplatePatchCollection with generated data
    """

    data = [{
        "path": path,
        "value": value,
        "op": "replace"
    }]
    collection = cluster_templatepatch_model.ClusterTemplatePatchCollection
    return collection.from_dict(data)


def cluster_template_remove_patch_data(path):
    """Generates ClusterTemplate patch data by removing value

    :param path: path to remove
    :returns: ClusterTemplatePatchCollection with generated data
    """

    data = [{
        "path": path,
        "op": "remove"
    }]
    collection = cluster_templatepatch_model.ClusterTemplatePatchCollection
    return collection.from_dict(data)


def cluster_template_name_patch_data(name=data_utils.rand_name('cluster')):
    """Generates random cluster_template patch data

    :param name: name to replace in patch
    :returns: ClusterTemplatePatchCollection with generated data
    """

    data = [{
        "path": "/name",
        "value": name,
        "op": "replace"
    }]
    collection = cluster_templatepatch_model.ClusterTemplatePatchCollection
    return collection.from_dict(data)


def cluster_template_flavor_patch_data(flavor=data_utils.rand_name('cluster')):
    """Generates random cluster_template patch data

    :param flavor: flavor to replace in patch
    :returns: ClusterTemplatePatchCollection with generated data
    """

    data = [{
        "path": "/flavor_id",
        "value": flavor,
        "op": "replace"
    }]
    collection = cluster_templatepatch_model.ClusterTemplatePatchCollection
    return collection.from_dict(data)


def cluster_template_data_with_valid_keypair_image_flavor():
    """Generates random clustertemplate data with valid data

    :returns: ClusterTemplateEntity with generated data
    """
    master_flavor = config.Config.master_flavor_id
    return cluster_template_data(keypair_id=config.Config.keypair_id,
                                 image_id=config.Config.image_id,
                                 flavor_id=config.Config.flavor_id,
                                 master_flavor_id=master_flavor)


def cluster_template_data_with_missing_image():
    """Generates random cluster_template data with missing image

    :returns: ClusterTemplateEntity with generated data
    """

    return cluster_template_data(
        keypair_id=config.Config.keypair_id,
        flavor_id=config.Config.flavor_id,
        master_flavor_id=config.Config.master_flavor_id)


def cluster_template_data_with_missing_flavor():
    """Generates random cluster_template data with missing flavor

    :returns: ClusterTemplateEntity with generated data
    """

    return cluster_template_data(keypair_id=config.Config.keypair_id,
                                 image_id=config.Config.image_id)


def cluster_template_data_with_missing_keypair():
    """Generates random cluster_template data with missing keypair

    :returns: ClusterTemplateEntity with generated data
    """

    return cluster_template_data(
        image_id=config.Config.image_id,
        flavor_id=config.Config.flavor_id,
        master_flavor_id=config.Config.master_flavor_id)


def cluster_template_valid_data_with_specific_coe(coe):
    """Generates random cluster_template data with valid keypair and image

    :param coe: coe
    :returns: ClusterTemplateEntity with generated data
    """

    return cluster_template_data(keypair_id=config.Config.keypair_id,
                                 image_id=config.Config.image_id, coe=coe)


def valid_swarm_mode_cluster_template(is_public=False):
    """Generates a valid swarm-mode cluster_template with valid data

    :returns: ClusterTemplateEntity with generated data
    """
    master_flavor_id = config.Config.master_flavor_id
    return cluster_template_data(image_id=config.Config.image_id,
                                 flavor_id=config.Config.flavor_id,
                                 public=is_public,
                                 dns_nameserver=config.Config.dns_nameserver,
                                 master_flavor_id=master_flavor_id,
                                 coe="swarm-mode",
                                 cluster_distro=None,
                                 external_network_id=config.Config.nic_id,
                                 http_proxy=None, https_proxy=None,
                                 no_proxy=None, network_driver=None,
                                 volume_driver=None, labels={},
                                 tls_disabled=False)


def cluster_data(name=data_utils.rand_name('cluster'),
                 cluster_template_id=data_utils.rand_uuid(),
                 node_count=random_int(1, 5), discovery_url=gen_random_ip(),
                 create_timeout=random_int(1, 30),
                 master_count=random_int(1, 5)):
    """Generates random cluster data

    cluster_template_id cannot be random for the cluster to be valid due to
    validations for the presence of clustertemplate prior to clustertemplate
    creation.

    :param name: cluster name (must be unique)
    :param cluster_template_id: clustertemplate unique id (must already exist)
    :param node_count: number of agents for cluster
    :param discovery_url: url provided for node discovery
    :param create_timeout: timeout in minutes for cluster create
    :param master_count: number of master nodes for the cluster
    :returns: ClusterEntity with generated data
    """

    data = {
        "name": name,
        "cluster_template_id": cluster_template_id,
        "keypair": config.Config.keypair_id,
        "node_count": node_count,
        "discovery_url": None,
        "create_timeout": create_timeout,
        "master_count": master_count
    }
    model = cluster_model.ClusterEntity.from_dict(data)

    return model


def valid_cluster_data(cluster_template_id,
                       name=data_utils.rand_name('cluster'),
                       node_count=1, master_count=1, create_timeout=None):
    """Generates random cluster data with valid

    :param cluster_template_id: clustertemplate unique id that already exists
    :param name: cluster name (must be unique)
    :param node_count: number of agents for cluster
    :returns: ClusterEntity with generated data
    """

    return cluster_data(cluster_template_id=cluster_template_id, name=name,
                        master_count=master_count, node_count=node_count,
                        create_timeout=create_timeout)


def cluster_name_patch_data(name=data_utils.rand_name('cluster')):
    """Generates random clustertemplate patch data

    :param name: name to replace in patch
    :returns: ClusterPatchCollection with generated data
    """

    data = [{
        "path": "/name",
        "value": name,
        "op": "replace"
    }]
    return clusterpatch_model.ClusterPatchCollection.from_dict(data)


def cluster_api_addy_patch_data(address='0.0.0.0'):
    """Generates random cluster patch data

    :param name: name to replace in patch
    :returns: ClusterPatchCollection with generated data
    """

    data = [{
        "path": "/api_address",
        "value": address,
        "op": "replace"
    }]
    return clusterpatch_model.ClusterPatchCollection.from_dict(data)


def cluster_node_count_patch_data(node_count=2):
    """Generates random cluster patch data

    :param name: name to replace in patch
    :returns: ClusterPatchCollection with generated data
    """

    data = [{
        "path": "/node_count",
        "value": node_count,
        "op": "replace"
    }]
    return clusterpatch_model.ClusterPatchCollection.from_dict(data)



# Copyright (c) 2018 European Organization for Nuclear Research.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest.mock import patch

from magnum.common import exception
from magnum.drivers.k8s_fedora_atomic_v1 import driver
from magnum.tests.unit.db import base
from magnum.tests.unit.objects import utils as obj_utils


class K8sFedoraAtomicV1DriverTest(base.DbTestCase):

    def setUp(self):
        super(K8sFedoraAtomicV1DriverTest, self).setUp()
        self.driver = driver.Driver()
        self.cluster_template = obj_utils.create_test_cluster_template(
            self.context, uuid='94889aa4-e686-11e9-81b4-2a2ae2dbcce4',
            name='test_2', id=2, labels={'kube_tag': 'v1.14.7'},
            image_id='test-image2')
        self.cluster_obj = obj_utils.create_test_cluster(
            self.context, name='cluster_example_A', image_id='test-image1')
        self.cluster_obj.refresh()
        self.nodegroup_obj = obj_utils.create_test_nodegroup(
            self.context, name='test_ng', cluster_id=self.cluster_obj.uuid,
            uuid='27e3153e-d5bf-4b7e-b517-fb518e17f34c',
            project_id=self.cluster_obj.project_id, is_default=False,
            image_id='test-image1')
        self.nodegroup_obj.refresh()

    @patch('magnum.common.keystone.KeystoneClientV3')
    @patch('magnum.common.clients.OpenStackClients')
    def test_upgrade_default_worker_ng(self, mock_osc, mock_keystone):
        mock_keystone.is_octavia_enabled.return_value = False
        def_ng = self.cluster_obj.default_ng_worker
        self.driver.upgrade_cluster(self.context, self.cluster_obj,
                                    self.cluster_template, 1, def_ng)
        # make sure that the kube_tag is reflected correctly to the cluster
        # and the default nodegroups.
        self.assertEqual(self.cluster_template.labels['kube_tag'],
                         self.cluster_obj.labels['kube_tag'])
        self.assertEqual(self.cluster_template.labels['kube_tag'],
                         self.cluster_obj.default_ng_master.labels['kube_tag'])
        self.assertEqual(self.cluster_template.labels['kube_tag'],
                         def_ng.labels['kube_tag'])
        # make sure that the image from the cluster
        # template is NOT set to the default nodegroups.
        self.assertEqual('test-image1',
                         self.cluster_obj.default_ng_master.image_id)
        self.assertEqual('test-image1', def_ng.image_id)
        # check that the non-default nodegroup was not changed
        self.assertNotIn('kube_tag', self.nodegroup_obj.labels)
        self.assertEqual('test-image1', self.nodegroup_obj.image_id)

    @patch('magnum.common.keystone.KeystoneClientV3')
    @patch('magnum.common.clients.OpenStackClients')
    def test_upgrade_default_master_ng(self, mock_osc, mock_keystone):
        mock_keystone.is_octavia_enabled.return_value = False
        def_ng = self.cluster_obj.default_ng_master
        self.driver.upgrade_cluster(self.context, self.cluster_obj,
                                    self.cluster_template, 1, def_ng)
        # make sure that the kube_tag is reflected correctly to the cluster
        # and the default nodegroups.
        self.assertEqual(self.cluster_template.labels['kube_tag'],
                         self.cluster_obj.labels['kube_tag'])
        self.assertEqual(self.cluster_template.labels['kube_tag'],
                         self.cluster_obj.default_ng_worker.labels['kube_tag'])
        self.assertEqual(self.cluster_template.labels['kube_tag'],
                         def_ng.labels['kube_tag'])
        # make sure that the image from the cluster
        # template is NOT set to the default nodegroups.
        self.assertEqual('test-image1',
                         self.cluster_obj.default_ng_worker.image_id)
        self.assertEqual('test-image1', def_ng.image_id)
        # check that the non-default nodegroup was not changed
        self.assertNotIn('kube_tag', self.nodegroup_obj.labels)
        self.assertEqual('test-image1', self.nodegroup_obj.image_id)

    @patch('magnum.common.keystone.KeystoneClientV3')
    @patch('magnum.common.clients.OpenStackClients')
    def test_upgrade_non_default_ng(self, mock_osc, mock_keystone):
        mock_keystone.is_octavia_enabled.return_value = False
        self.driver.upgrade_cluster(self.context, self.cluster_obj,
                                    self.cluster_template, 1,
                                    self.nodegroup_obj)
        # check that the cluster and default nodegroups were not changed
        self.assertNotIn('kube_tag', self.cluster_obj.labels)
        self.assertNotIn('kube_tag', self.cluster_obj.default_ng_master.labels)
        self.assertNotIn('kube_tag', self.cluster_obj.default_ng_worker.labels)
        # make sure that the image from the cluster template
        # is not reflected to the default nodegroups.
        self.assertEqual('test-image1',
                         self.cluster_obj.default_ng_master.image_id)
        self.assertEqual('test-image1',
                         self.cluster_obj.default_ng_worker.image_id)
        # check that the non-default nodegroup reflects the cluster template.
        self.assertEqual(self.cluster_template.labels['kube_tag'],
                         self.nodegroup_obj.labels['kube_tag'])
        self.assertEqual('test-image1',
                         self.nodegroup_obj.image_id)

    @patch('magnum.common.keystone.KeystoneClientV3')
    @patch('magnum.common.clients.OpenStackClients')
    def test_downgrade_not_supported(self, mock_osc, mock_keystone):
        # Scenario, a user creates a nodegroup with kube_tag
        # greater that the one set in cluster's template. Check
        # that downgrading is not supported.
        self.nodegroup_obj.labels = {'kube_tag': 'v1.14.7'}
        self.nodegroup_obj.save()
        self.cluster_template.labels = {'kube_tag': 'v1.14.3'}
        self.cluster_template.save()
        mock_keystone.is_octavia_enabled.return_value = False
        self.assertRaises(exception.InvalidVersion,
                          self.driver.upgrade_cluster, self.context,
                          self.cluster_obj, self.cluster_template, 1,
                          self.nodegroup_obj)

    @patch('magnum.common.keystone.KeystoneClientV3')
    @patch('magnum.common.clients.OpenStackClients')
    def test_invalid_ct(self, mock_osc, mock_keystone):
        # Scenario, a user creates a nodegroup with kube_tag
        # greater that the one set in cluster's template. Check
        # that downgrading is not supported.
        self.cluster_template.labels = {}
        self.cluster_template.save()
        mock_keystone.is_octavia_enabled.return_value = False
        self.assertRaises(exception.InvalidClusterTemplateForUpgrade,
                          self.driver.upgrade_cluster, self.context,
                          self.cluster_obj, self.cluster_template, 1,
                          self.nodegroup_obj)

    @patch('magnum.common.keystone.KeystoneClientV3')
    @patch('magnum.common.clients.OpenStackClients')
    def test_ca_rotate_not_supported(self, mock_osc, mock_keystone):
        self.cluster_template.cluster_distro = 'fedora-atomic'
        self.cluster_template.save()
        mock_keystone.is_octavia_enabled.return_value = False
        self.assertRaises(exception.NotSupported,
                          self.driver.rotate_ca_certificate,
                          self.context,
                          self.cluster_obj)



# Copyright (c) 2018 European Organization for Nuclear Research.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import functools

from heatclient import exc
from oslo_log import log as logging
import six

from magnum.common import exception
from magnum.common import profiler
import magnum.conf
from magnum.drivers.common import driver
from magnum.i18n import _
from magnum.objects import fields

CONF = magnum.conf.CONF

LOG = logging.getLogger(__name__)


# TODO(ttsiouts): notifications about nodegroup operations will be
#                 added in later commit.


ALLOWED_NODEGROUP_STATES = (
    fields.ClusterStatus.CREATE_COMPLETE,
    fields.ClusterStatus.UPDATE_COMPLETE,
    fields.ClusterStatus.UPDATE_IN_PROGRESS,
    fields.ClusterStatus.UPDATE_FAILED,
    fields.ClusterStatus.RESUME_COMPLETE,
    fields.ClusterStatus.RESTORE_COMPLETE,
    fields.ClusterStatus.ROLLBACK_COMPLETE,
    fields.ClusterStatus.SNAPSHOT_COMPLETE,
    fields.ClusterStatus.CHECK_COMPLETE,
    fields.ClusterStatus.ADOPT_COMPLETE
)


def allowed_operation(func):
    @functools.wraps(func)
    def wrapper(self, context, cluster, nodegroup, *args, **kwargs):
        # Before we begin we need to check the status
        # of the cluster. If the cluster is in a status
        # that does not allow nodegroup creation we just
        # fail.
        if ('status' in nodegroup
                and nodegroup.status not in ALLOWED_NODEGROUP_STATES):
            operation = _(
                '%(fname)s when nodegroup status is "%(status)s"'
            ) % {'fname': func.__name__, 'status': cluster.status}
            raise exception.NotSupported(operation=operation)
        return func(self, context, cluster, nodegroup, *args, **kwargs)

    return wrapper


@profiler.trace_cls("rpc")
class Handler(object):

    @allowed_operation
    def nodegroup_create(self, context, cluster, nodegroup):
        LOG.debug("nodegroup_conductor nodegroup_create")
        cluster.status = fields.ClusterStatus.UPDATE_IN_PROGRESS
        cluster.save()
        nodegroup.status = fields.ClusterStatus.CREATE_IN_PROGRESS
        nodegroup.create()

        try:
            cluster_driver = driver.Driver.get_driver_for_cluster(context,
                                                                  cluster)
            cluster_driver.create_nodegroup(context, cluster, nodegroup)
            nodegroup.save()
        except Exception as e:
            nodegroup.status = fields.ClusterStatus.CREATE_FAILED
            nodegroup.status_reason = six.text_type(e)
            nodegroup.save()
            cluster.status = fields.ClusterStatus.UPDATE_FAILED
            cluster.save()
            if isinstance(e, exc.HTTPBadRequest):
                e = exception.InvalidParameterValue(message=six.text_type(e))
                raise e
            raise
        return nodegroup

    @allowed_operation
    def nodegroup_update(self, context, cluster, nodegroup):
        LOG.debug("nodegroup_conductor nodegroup_update")
        cluster.status = fields.ClusterStatus.UPDATE_IN_PROGRESS
        cluster.save()
        nodegroup.status = fields.ClusterStatus.UPDATE_IN_PROGRESS

        try:
            cluster_driver = driver.Driver.get_driver_for_cluster(context,
                                                                  cluster)
            cluster_driver.update_nodegroup(context, cluster, nodegroup)
            nodegroup.save()
        except Exception as e:
            nodegroup.status = fields.ClusterStatus.UPDATE_FAILED
            nodegroup.status_reason = six.text_type(e)
            nodegroup.save()
            cluster.status = fields.ClusterStatus.UPDATE_FAILED
            cluster.save()
            if isinstance(e, exc.HTTPBadRequest):
                e = exception.InvalidParameterValue(message=six.text_type(e))
                raise e
            raise

        return nodegroup

    def nodegroup_delete(self, context, cluster, nodegroup):
        LOG.debug("nodegroup_conductor nodegroup_delete")
        cluster.status = fields.ClusterStatus.UPDATE_IN_PROGRESS
        cluster.save()
        nodegroup.status = fields.ClusterStatus.DELETE_IN_PROGRESS

        try:
            cluster_driver = driver.Driver.get_driver_for_cluster(context,
                                                                  cluster)
            cluster_driver.delete_nodegroup(context, cluster, nodegroup)
        except exc.HTTPNotFound:
            LOG.info('The nodegroup %s was not found during nodegroup'
                     ' deletion.', nodegroup.uuid)
            try:
                nodegroup.destroy()
            except exception.NodeGroupNotFound:
                LOG.info('The nodegroup %s has been deleted by others.',
                         nodegroup.uuid)
            return None
        except exc.HTTPConflict:
            raise exception.NgOperationInProgress(nodegroup=nodegroup.name)
        except Exception as e:
            nodegroup.status = fields.ClusterStatus.DELETE_FAILED
            nodegroup.status_reason = six.text_type(e)
            nodegroup.save()
            cluster.status = fields.ClusterStatus.UPDATE_FAILED
            cluster.save()
            raise
        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

from magnum.i18n import _

trust_group = cfg.OptGroup(name='trust',
                           title='Trustee options for the magnum services')

trust_opts = [
    cfg.BoolOpt('cluster_user_trust',
                default=False,
                help=_('This setting controls whether to assign a trust to'
                       ' the cluster user or not. You will need to set it to'
                       ' True for clusters with volume_driver=cinder or'
                       ' registry_enabled=true in the underlying cluster'
                       ' template to work. This is a potential security risk'
                       ' since the trust gives instances OpenStack API access'
                       " to the cluster's project. Note that this setting"
                       ' does not affect per-cluster trusts assigned to the'
                       ' Magnum service user.')),
    cfg.StrOpt('trustee_domain_id',
               help=_('Id of the domain to create trustee for clusters')),
    cfg.StrOpt('trustee_domain_name',
               help=_('Name of the domain to create trustee for s')),
    cfg.StrOpt('trustee_domain_admin_id',
               help=_('Id of the admin with roles sufficient to manage users'
                      ' in the trustee_domain')),
    cfg.StrOpt('trustee_domain_admin_name',
               help=_('Name of the admin with roles sufficient to manage users'
                      ' in the trustee_domain')),
    cfg.StrOpt('trustee_domain_admin_domain_id',
               help=_('Id of the domain admin user\'s domain.'
                      ' trustee_domain_id is used by default')),
    cfg.StrOpt('trustee_domain_admin_domain_name',
               help=_('Name of the domain admin user\'s domain.'
                      ' trustee_domain_name is used by default')),
    cfg.StrOpt('trustee_domain_admin_password', secret=True,
               help=_('Password of trustee_domain_admin')),
    cfg.ListOpt('roles',
                default=[],
                help=_('The roles which are delegated to the trustee '
                       'by the trustor')),
    cfg.StrOpt('trustee_keystone_interface',
               default='public',
               help=_('Auth interface used by instances/trustee')),
    cfg.StrOpt('trustee_keystone_region_name',
               help=_('Region in Identity service catalog to use for '
                      'communication with the OpenStack service.'))
]


def register_opts(conf):
    conf.register_group(trust_group)
    conf.register_opts(trust_opts, group=trust_group)


def list_opts():
    return {
        trust_group: trust_opts
    }



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneauth1.access import access as ka_access
from keystoneauth1 import exceptions as ka_exception
from keystoneauth1.identity import access as ka_access_plugin
from keystoneauth1.identity import v3 as ka_v3
from keystoneauth1 import loading as ka_loading
import keystoneclient.exceptions as kc_exception
from keystoneclient.v3 import client as kc_v3
from oslo_log import log as logging

from magnum.common import exception
import magnum.conf
from magnum.conf import keystone as ksconf
from magnum.i18n import _

CONF = magnum.conf.CONF
LOG = logging.getLogger(__name__)


class KeystoneClientV3(object):
    """Keystone client wrapper so we can encapsulate logic in one place."""

    def __init__(self, context):
        self.context = context
        self._client = None
        self._domain_admin_auth = None
        self._domain_admin_session = None
        self._domain_admin_client = None
        self._trustee_domain_id = None
        self._session = None

    @property
    def auth_url(self):
        # FIXME(pauloewerton): auth_url should be retrieved from keystone_auth
        # section by default
        conf = CONF[ksconf.CFG_LEGACY_GROUP]
        auth_uri = (getattr(conf, 'www_authenticate_uri', None) or
                    getattr(conf, 'auth_uri', None))
        if auth_uri:
            auth_uri = auth_uri.replace('v2.0', 'v3')
        return auth_uri

    @property
    def auth_token(self):
        return self.session.get_token()

    @property
    def session(self):
        if self._session:
            return self._session
        auth = self._get_auth()
        session = self._get_session(auth)
        self._session = session
        return session

    def _get_session(self, auth):
        session = ka_loading.load_session_from_conf_options(
            CONF, ksconf.CFG_GROUP, auth=auth)
        return session

    def _get_auth(self):
        if self.context.auth_token_info:
            access_info = ka_access.create(body=self.context.auth_token_info,
                                           auth_token=self.context.auth_token)
            auth = ka_access_plugin.AccessInfoPlugin(access_info)
        elif self.context.auth_token:
            auth = ka_v3.Token(auth_url=self.auth_url,
                               token=self.context.auth_token)
        elif self.context.trust_id:
            auth_info = {
                'auth_url': self.auth_url,
                'username': self.context.user_name,
                'password': self.context.password,
                'user_domain_id': self.context.user_domain_id,
                'user_domain_name': self.context.user_domain_name,
                'trust_id': self.context.trust_id
            }

            auth = ka_v3.Password(**auth_info)
        elif self.context.is_admin:
            try:
                auth = ka_loading.load_auth_from_conf_options(
                    CONF, ksconf.CFG_GROUP)
            except ka_exception.MissingRequiredOptions:
                auth = self._get_legacy_auth()
        else:
            msg = ('Keystone API connection failed: no password, '
                   'trust_id or token found.')
            LOG.error(msg)
            raise exception.AuthorizationFailure(client='keystone',
                                                 message='reason %s' % msg)

        return auth

    def _get_legacy_auth(self):
        LOG.warning('Auth plugin and its options for service user '
                    'must be provided in [%(new)s] section. '
                    'Using values from [%(old)s] section is '
                    'deprecated.', {'new': ksconf.CFG_GROUP,
                                    'old': ksconf.CFG_LEGACY_GROUP})

        conf = getattr(CONF, ksconf.CFG_LEGACY_GROUP)

        # FIXME(htruta, pauloewerton): Conductor layer does not have
        # new v3 variables, such as project_name and project_domain_id.
        # The use of admin_* variables is related to Identity API v2.0,
        # which is now deprecated. We should also stop using hard-coded
        # domain info, as well as variables that refer to `tenant`,
        # as they are also v2 related.
        auth = ka_v3.Password(auth_url=self.auth_url,
                              username=conf.admin_user,
                              password=conf.admin_password,
                              project_name=conf.admin_tenant_name,
                              project_domain_id='default',
                              user_domain_id='default')
        return auth

    @property
    def client(self):
        if self._client:
            return self._client
        client = kc_v3.Client(session=self.session,
                              trust_id=self.context.trust_id)
        self._client = client
        return client

    @property
    def domain_admin_auth(self):
        user_domain_id = (
            CONF.trust.trustee_domain_admin_domain_id or
            CONF.trust.trustee_domain_id
        )
        user_domain_name = (
            CONF.trust.trustee_domain_admin_domain_name or
            CONF.trust.trustee_domain_name
        )
        if not self._domain_admin_auth:
            self._domain_admin_auth = ka_v3.Password(
                auth_url=self.auth_url,
                user_id=CONF.trust.trustee_domain_admin_id,
                username=CONF.trust.trustee_domain_admin_name,
                user_domain_id=user_domain_id,
                user_domain_name=user_domain_name,
                domain_id=CONF.trust.trustee_domain_id,
                domain_name=CONF.trust.trustee_domain_name,
                password=CONF.trust.trustee_domain_admin_password)
        return self._domain_admin_auth

    @property
    def domain_admin_session(self):
        if not self._domain_admin_session:
            session = ka_loading.session.Session().load_from_options(
                auth=self.domain_admin_auth,
                insecure=CONF[ksconf.CFG_LEGACY_GROUP].insecure,
                cacert=CONF[ksconf.CFG_LEGACY_GROUP].cafile,
                key=CONF[ksconf.CFG_LEGACY_GROUP].keyfile,
                cert=CONF[ksconf.CFG_LEGACY_GROUP].certfile)
            self._domain_admin_session = session
        return self._domain_admin_session

    @property
    def domain_admin_client(self):
        if not self._domain_admin_client:
            self._domain_admin_client = kc_v3.Client(
                session=self.domain_admin_session
            )
        return self._domain_admin_client

    @property
    def trustee_domain_id(self):
        if not self._trustee_domain_id:
            try:
                access = self.domain_admin_auth.get_access(
                    self.domain_admin_session
                )
            except kc_exception.Unauthorized:
                msg = "Keystone client authentication failed"
                LOG.error(msg)
                raise exception.AuthorizationFailure(client='keystone',
                                                     message='reason: %s' %
                                                             msg)

            self._trustee_domain_id = access.domain_id

        return self._trustee_domain_id

    def create_trust(self, trustee_user):
        trustor_user_id = self.session.get_user_id()
        trustor_project_id = self.session.get_project_id()

        # inherit the role of the trustor, unless set CONF.trust.roles
        if CONF.trust.roles:
            roles = CONF.trust.roles
        else:
            roles = self.context.roles

        try:
            trust = self.client.trusts.create(
                trustor_user=trustor_user_id,
                project=trustor_project_id,
                trustee_user=trustee_user,
                impersonation=True,
                delegation_depth=0,
                role_names=roles)
        except Exception:
            LOG.exception('Failed to create trust')
            raise exception.TrustCreateFailed(
                trustee_user_id=trustee_user)
        return trust

    def delete_trust(self, context, cluster):
        if cluster.trust_id is None:
            return

        # Trust can only be deleted by the user who creates it. So when
        # other users in the same project want to delete the cluster, we need
        # use the trustee which can impersonate the trustor to delete the
        # trust.
        if context.user_id == cluster.user_id:
            client = self.client
        else:
            auth = ka_v3.Password(auth_url=self.auth_url,
                                  user_id=cluster.trustee_user_id,
                                  password=cluster.trustee_password,
                                  trust_id=cluster.trust_id)

            sess = ka_loading.session.Session().load_from_options(
                auth=auth,
                insecure=CONF[ksconf.CFG_LEGACY_GROUP].insecure,
                cacert=CONF[ksconf.CFG_LEGACY_GROUP].cafile,
                key=CONF[ksconf.CFG_LEGACY_GROUP].keyfile,
                cert=CONF[ksconf.CFG_LEGACY_GROUP].certfile)
            client = kc_v3.Client(session=sess)
        try:
            client.trusts.delete(cluster.trust_id)
        except kc_exception.NotFound:
            pass
        except Exception:
            LOG.exception('Failed to delete trust')
            raise exception.TrustDeleteFailed(trust_id=cluster.trust_id)

    def create_trustee(self, username, password):
        domain_id = self.trustee_domain_id
        try:
            user = self.domain_admin_client.users.create(
                name=username,
                password=password,
                domain=domain_id)
        except Exception:
            LOG.exception('Failed to create trustee')
            raise exception.TrusteeCreateFailed(username=username,
                                                domain_id=domain_id)
        return user

    def delete_trustee(self, trustee_id):
        try:
            self.domain_admin_client.users.delete(trustee_id)
        except kc_exception.NotFound:
            pass
        except Exception:
            LOG.exception('Failed to delete trustee')
            raise exception.TrusteeDeleteFailed(trustee_id=trustee_id)

    def get_validate_region_name(self, region_name):
        if region_name is None:
            message = _("region_name needs to be configured in magnum.conf")
            raise exception.InvalidParameterValue(message)
        """matches the region of a public endpoint for the Keystone
        service."""
        try:
            regions = self.client.regions.list()
        except kc_exception.NotFound:
            pass
        except Exception:
            LOG.exception('Failed to list regions')
            raise exception.RegionsListFailed()
        region_list = []
        for region in regions:
            region_list.append(region.id)
        if region_name not in region_list:
            raise exception.InvalidParameterValue(_(
                'region_name %(region_name)s is invalid, '
                'expecting a region_name in %(region_name_list)s.') % {
                    'region_name': region_name,
                    'region_name_list': '/'.join(
                        region_list + ['unspecified'])})
        return region_name


def is_octavia_enabled():
    """Check if Octavia service is deployed in the cloud.

    Octavia is already an official LBaaS solution for Openstack
    (https://governance.openstack.org/tc/reference/projects/octavia.html) and
    will deprecate the neutron-lbaas extension starting from Queens release.

    We use Octavia instead of Neutron LBaaS API for load balancing
    functionality for k8s cluster if Octavia service is deployed and enabled
    in the cloud.
    """
    # Put the import here to avoid circular importing.
    from magnum.common import context
    admin_context = context.make_admin_context()
    keystone = KeystoneClientV3(admin_context)

    try:
        octavia_svc = keystone.client.services.list(type='load-balancer')
    except Exception:
        LOG.exception('Failed to list services')
        raise exception.ServicesListFailed()

    # Always assume there is only one load balancing service configured.
    if octavia_svc and octavia_svc[0].enabled:
        return True

    return False



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

from magnum.i18n import _

cinder_group = cfg.OptGroup(
    name='cinder',
    title='Options for the Cinder configuration')

cinder_client_group = cfg.OptGroup(
    name='cinder_client',
    title='Options for the Cinder client')

cinder_opts = [
    cfg.StrOpt('default_docker_volume_type',
               default='',
               help=_('The default docker volume_type to use for volumes '
                      'used for docker storage. To use the cinder volumes '
                      'for docker storage, you need to select a default '
                      'value. Otherwise, Magnum will select random one from '
                      'Cinder volume type list.')),
    cfg.StrOpt('default_etcd_volume_type',
               default='',
               help=_('The default etcd volume_type to use for volumes '
                      'used for etcd storage. To use the cinder volumes '
                      'for etcd storage, you need to select a default '
                      'value. Otherwise, Magnum will select random one from '
                      'Cinder volume type list.')),
    cfg.StrOpt('default_boot_volume_type',
               default='',
               help=_('The default boot volume_type to use for volumes '
                      'used for VM of COE. To use the cinder volumes '
                      'for VM of COE, you need to select a default '
                      'value. Otherwise, Magnum will select random one from '
                      'Cinder volume type list.')),
    cfg.IntOpt('default_boot_volume_size',
               default=0,
               help=_('The default volume size to use for volumes '
                      'used for VM of COE.'))
]

cinder_client_opts = [
    cfg.StrOpt('region_name',
               help=_('Region in Identity service catalog to use for '
                      'communication with the OpenStack service.')),
    cfg.StrOpt('endpoint_type',
               default='publicURL',
               help=_('Type of endpoint in Identity service catalog to use '
                      'for communication with the OpenStack service.')),
    cfg.StrOpt('api_version',
               default='2',
               help=_('Version of Cinder API to use in cinderclient.'))
]

common_security_opts = [
    cfg.StrOpt('ca_file',
               help=_('Optional CA cert file to use in SSL connections.')),
    cfg.StrOpt('cert_file',
               help=_('Optional PEM-formatted certificate chain file.')),
    cfg.StrOpt('key_file',
               help=_('Optional PEM-formatted file that contains the '
                      'private key.')),
    cfg.BoolOpt('insecure',
                default=False,
                help=_("If set, then the server's certificate will not "
                       "be verified."))]


def register_opts(conf):
    conf.register_group(cinder_group)
    conf.register_group(cinder_client_group)
    conf.register_opts(cinder_opts, group=cinder_group)
    conf.register_opts(cinder_client_opts, group=cinder_client_group)
    conf.register_opts(common_security_opts, group=cinder_client_group)


def list_opts():
    return {
        cinder_group: cinder_opts,
        cinder_client_group: cinder_client_opts
    }



# Copyright 2019 Catalyst Cloud Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo_config import cfg
from oslo_log import log as logging

from magnum.common import clients
from novaclient import exceptions as nova_exception

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


def get_ssh_key(context, keypair_ident):
    try:
        n_client = clients.OpenStackClients(context).nova()
        keypair = n_client.keypairs.get(keypair_ident)
        # no spaces or break lines at the end, single line string
        return keypair.public_key.strip()
    except nova_exception.NotFound:
        # we don't have a way to tell if the keypair doesn't
        # exist or the cluster is already creted
        return ""



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Add master_addresses to bay

Revision ID: 6f21dc998bb
Revises: 421102d1f2d2
Create Date: 2015-08-20 13:57:14.863292

"""

# revision identifiers, used by Alembic.
revision = '6f21dc998bb'
down_revision = '421102d1f2d2'

from alembic import op  # noqa: E402
from magnum.db.sqlalchemy import models  # noqa: E402
import sqlalchemy as sa  # noqa: E402


def upgrade():
    op.add_column(
        'bay',
        sa.Column('master_addresses',
                  models.JSONEncodedList(),
                  nullable=True)
    )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import strutils
from oslo_utils import uuidutils
from oslo_versionedobjects import fields

from magnum.common import exception
from magnum.db import api as dbapi
from magnum.objects import base
from magnum.objects import fields as m_fields


@base.MagnumObjectRegistry.register
class Federation(base.MagnumPersistentObject, base.MagnumObject,
                 base.MagnumObjectDictCompat):
    """Represents a Federation object.

    Version 1.0: Initial Version
    """

    VERSION = '1.0'

    dbapi = dbapi.get_instance()

    fields = {
        'id': fields.IntegerField(),
        'uuid': fields.UUIDField(nullable=True),
        'name': fields.StringField(nullable=True),
        'project_id': fields.StringField(nullable=True),
        'hostcluster_id': fields.StringField(nullable=True),
        'member_ids': fields.ListOfStringsField(nullable=True),
        'status': m_fields.FederationStatusField(nullable=True),
        'status_reason': fields.StringField(nullable=True),
        'properties': fields.DictOfStringsField(nullable=True)
    }

    @staticmethod
    def _from_db_object(federation, db_federation):
        """Converts a database entity to a formal object."""
        for field in federation.fields:
            federation[field] = db_federation[field]

        federation.obj_reset_changes()
        return federation

    @staticmethod
    def _from_db_object_list(db_objects, cls, context):
        """Converts a list of database entities to a list of formal objects."""
        return [Federation._from_db_object(cls(context), obj)
                for obj in db_objects]

    @base.remotable_classmethod
    def get(cls, context, federation_id):
        """Find a federation based on its id or uuid and return it.

        :param federation_id: the id *or* uuid of a federation.
        :param context: Security context
        :returns: a :class:`Federation` object.
        """
        if strutils.is_int_like(federation_id):
            return cls.get_by_id(context, federation_id)
        elif uuidutils.is_uuid_like(federation_id):
            return cls.get_by_uuid(context, federation_id)
        else:
            raise exception.InvalidIdentity(identity=federation_id)

    @base.remotable_classmethod
    def get_by_id(cls, context, federation_id):
        """Find a federation based on its integer id and return it.

        :param federation_id: the id of a federation.
        :param context: Security context
        :returns: a :class:`Federation` object.
        """
        db_federation = cls.dbapi.get_federation_by_id(context, federation_id)
        federation = Federation._from_db_object(cls(context), db_federation)
        return federation

    @base.remotable_classmethod
    def get_by_uuid(cls, context, uuid):
        """Find a federation based on uuid and return it.

        :param uuid: the uuid of a federation.
        :param context: Security context
        :returns: a :class:`Federation` object.
        """
        db_federation = cls.dbapi.get_federation_by_uuid(context, uuid)
        federation = Federation._from_db_object(cls(context), db_federation)
        return federation

    @base.remotable_classmethod
    def get_count_all(cls, context, filters=None):
        """Get count of matching federation.

        :param context: The security context
        :param filters: filter dict, can includes 'name', 'project_id',
                        'hostcluster_id', 'member_ids', 'status' (should be a
                        status list).
        :returns: Count of matching federation.
        """
        return cls.dbapi.get_federation_count_all(context, filters=filters)

    @base.remotable_classmethod
    def get_by_name(cls, context, name):
        """Find a federation based on name and return a Federation object.

        :param name: the logical name of a federation.
        :param context: Security context
        :returns: a :class:`Federation` object.
        """
        db_federation = cls.dbapi.get_federation_by_name(context, name)
        federation = Federation._from_db_object(cls(context), db_federation)
        return federation

    @base.remotable_classmethod
    def list(cls, context, limit=None, marker=None,
             sort_key=None, sort_dir=None, filters=None):
        """Return a list of Federation objects.

        :param context: Security context.
        :param limit: maximum number of resources to return in a single result.
        :param marker: pagination marker for large data sets.
        :param sort_key: column to sort results by.
        :param sort_dir: direction to sort. "asc" or "desc".
        :param filters: filter dict, can includes 'name', 'project_id',
                        'hostcluster_id', 'member_ids', 'status' (should be a
                        status list).
        :returns: a list of :class:`Federation` object.

        """
        db_federation = cls.dbapi.get_federation_list(context, limit=limit,
                                                      marker=marker,
                                                      sort_key=sort_key,
                                                      sort_dir=sort_dir,
                                                      filters=filters)
        return Federation._from_db_object_list(db_federation, cls, context)

    @base.remotable
    def create(self, context=None):
        """Create a Federation record in the DB.

        :param context: Security context. NOTE: This should only
                        be used internally by the indirection_api.
                        Unfortunately, RPC requires context as the first
                        argument, even though we don't use it.
                        A context should be set when instantiating the
                        object, e.g.: Federation(context)

        """
        values = self.obj_get_changes()
        db_federation = self.dbapi.create_federation(values)
        self._from_db_object(self, db_federation)

    @base.remotable
    def destroy(self, context=None):
        """Delete the Federation from the DB.

        :param context: Security context. NOTE: This should only
                        be used internally by the indirection_api.
                        Unfortunately, RPC requires context as the first
                        argument, even though we don't use it.
                        A context should be set when instantiating the
                        object, e.g.: Federation(context)
        """
        self.dbapi.destroy_federation(self.uuid)
        self.obj_reset_changes()

    @base.remotable
    def save(self, context=None):
        """Save updates to this Federation.

        Updates will be made column by column based on the result
        of self.what_changed().

        :param context: Security context. NOTE: This should only
                        be used internally by the indirection_api.
                        Unfortunately, RPC requires context as the first
                        argument, even though we don't use it.
                        A context should be set when instantiating the
                        object, e.g.: Federation(context)
        """
        updates = self.obj_get_changes()
        self.dbapi.update_federation(self.uuid, updates)

        self.obj_reset_changes()

    @base.remotable
    def refresh(self, context=None):
        """Load updates for this Federation.

        Loads a Federation with the same uuid from the database and
        checks for updated attributes. Updates are applied from
        the loaded Federation column by column, if there are any updates.

        :param context: Security context. NOTE: This should only
                        be used internally by the indirection_api.
                        Unfortunately, RPC requires context as the first
                        argument, even though we don't use it.
                        A context should be set when instantiating the
                        object, e.g.: Federation(context)
        """
        current = self.__class__.get_by_uuid(self._context, uuid=self.uuid)
        for field in self.fields:
            if self.obj_attr_is_set(field) and self[field] != current[field]:
                self[field] = current[field]



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Base classes for storage engines
"""

import abc

from oslo_config import cfg
from oslo_db import api as db_api
import six

from magnum.common import profiler


_BACKEND_MAPPING = {'sqlalchemy': 'magnum.db.sqlalchemy.api'}
IMPL = db_api.DBAPI.from_config(cfg.CONF, backend_mapping=_BACKEND_MAPPING,
                                lazy=True)


def get_instance():
    """Return a DB API instance."""
    return IMPL


@profiler.trace_cls("db")
@six.add_metaclass(abc.ABCMeta)
class Connection(object):
    """Base class for storage system connections."""

    @abc.abstractmethod
    def __init__(self):
        """Constructor."""

    @abc.abstractmethod
    def get_cluster_list(self, context, filters=None, limit=None,
                         marker=None, sort_key=None, sort_dir=None):
        """Get matching clusters.

        Return a list of the specified columns for all clusters that match the
        specified filters.

        :param context: The security context
        :param filters: Filters to apply. Defaults to None.

        :param limit: Maximum number of clusters to return.
        :param marker: the last item of the previous page; we return the next
                       result set.
        :param sort_key: Attribute by which results should be sorted.
        :param sort_dir: direction in which results should be sorted.
                         (asc, desc)
        :returns: A list of tuples of the specified columns.
        """

    @abc.abstractmethod
    def create_cluster(self, values):
        """Create a new cluster.

        :param values: A dict containing several items used to identify
                       and track the cluster, and several dicts which are
                       passed into the Drivers when managing this cluster.
                        For example:

                       ::

                        {
                         'uuid': uuidutils.generate_uuid(),
                         'name': 'example',
                         'type': 'virt'
                        }
        :returns: A cluster.
        """

    @abc.abstractmethod
    def get_cluster_by_id(self, context, cluster_id):
        """Return a cluster.

        :param context: The security context
        :param cluster_id: The id of a cluster.
        :returns: A cluster.
        """

    @abc.abstractmethod
    def get_cluster_by_uuid(self, context, cluster_uuid):
        """Return a cluster.

        :param context: The security context
        :param cluster_uuid: The uuid of a cluster.
        :returns: A cluster.
        """

    @abc.abstractmethod
    def get_cluster_by_name(self, context, cluster_name):
        """Return a cluster.

        :param context: The security context
        :param cluster_name: The name of a cluster.
        :returns: A cluster.
        """

    @abc.abstractmethod
    def get_cluster_stats(self, context, project_id):
        """Return clusters stats for the given project.

        :param context: The security context
        :param project_id: The project id.
        :returns: clusters, nodes count.
        """

    @abc.abstractmethod
    def get_cluster_count_all(self, context, filters=None):
        """Get count of matching clusters.

        :param context: The security context
        :param filters: Filters to apply. Defaults to None.
        :returns: Count of matching clusters.
        """

    @abc.abstractmethod
    def destroy_cluster(self, cluster_id):
        """Destroy a cluster and all associated interfaces.

        :param cluster_id: The id or uuid of a cluster.
        """

    @abc.abstractmethod
    def update_cluster(self, cluster_id, values):
        """Update properties of a cluster.

        :param cluster_id: The id or uuid of a cluster.
        :returns: A cluster.
        :raises: ClusterNotFound
        """

    @abc.abstractmethod
    def get_cluster_template_list(self, context, filters=None,
                                  limit=None, marker=None, sort_key=None,
                                  sort_dir=None):
        """Get matching ClusterTemplates.

        Return a list of the specified columns for all ClusterTemplates that
        match the specified filters.

        :param context: The security context
        :param filters: Filters to apply. Defaults to None.

        :param limit: Maximum number of ClusterTemplates to return.
        :param marker: the last item of the previous page; we return the next
                       result set.
        :param sort_key: Attribute by which results should be sorted.
        :param sort_dir: direction in which results should be sorted.
                         (asc, desc)
        :returns: A list of tuples of the specified columns.
        """

    @abc.abstractmethod
    def create_cluster_template(self, values):
        """Create a new ClusterTemplate.

        :param values: A dict containing several items used to identify
                       and track the ClusterTemplate, and several dicts which
                       are passed into the Drivers when managing this
                       ClusterTemplate.
                       For example:

                       ::

                        {
                         'uuid': uuidutils.generate_uuid(),
                         'name': 'example',
                         'type': 'virt'
                        }
        :returns: A ClusterTemplate.
        """

    @abc.abstractmethod
    def get_cluster_template_by_id(self, context, cluster_template_id):
        """Return a ClusterTemplate.

        :param context: The security context
        :param cluster_template_id: The id of a ClusterTemplate.
        :returns: A ClusterTemplate.
        """

    @abc.abstractmethod
    def get_cluster_template_by_uuid(self, context, cluster_template_uuid):
        """Return a ClusterTemplate.

        :param context: The security context
        :param cluster_template_uuid: The uuid of a ClusterTemplate.
        :returns: A ClusterTemplate.
        """

    @abc.abstractmethod
    def get_cluster_template_by_name(self, context, cluster_template_name):
        """Return a ClusterTemplate.

        :param context: The security context
        :param cluster_template_name: The name of a ClusterTemplate.
        :returns: A ClusterTemplate.
        """

    @abc.abstractmethod
    def destroy_cluster_template(self, cluster_template_id):
        """Destroy a ClusterTemplate and all associated interfaces.

        :param cluster_template_id: The id or uuid of a ClusterTemplate.
        """

    @abc.abstractmethod
    def update_cluster_template(self, cluster_template_id, values):
        """Update properties of a ClusterTemplate.

        :param cluster_template_id: The id or uuid of a ClusterTemplate.
        :returns: A ClusterTemplate.
        :raises: ClusterTemplateNotFound
        """

    @abc.abstractmethod
    def create_x509keypair(self, values):
        """Create a new x509keypair.

        :param values: A dict containing several items used to identify
                       and track the x509keypair, and several dicts which
                       are passed into the Drivers when managing this
                       x509keypair. For example:

                       ::

                        {
                         'uuid': uuidutils.generate_uuid(),
                         'certificate': 'AAA...',
                         'private_key': 'BBB...',
                         'private_key_passphrase': 'CCC...',
                         'intermediates': 'DDD...',
                        }
        :returns: A X509KeyPair.
        """

    @abc.abstractmethod
    def get_x509keypair_by_id(self, context, x509keypair_id):
        """Return a x509keypair.

        :param context: The security context
        :param x509keypair_id: The id of a x509keypair.
        :returns: A x509keypair.
        """

    @abc.abstractmethod
    def get_x509keypair_by_uuid(self, context, x509keypair_uuid):
        """Return a x509keypair.

        :param context: The security context
        :param x509keypair_uuid: The uuid of a x509keypair.
        :returns: A x509keypair.
        """

    @abc.abstractmethod
    def destroy_x509keypair(self, x509keypair_id):
        """Destroy a x509keypair.

        :param x509keypair_id: The id or uuid of a x509keypair.
        """

    @abc.abstractmethod
    def update_x509keypair(self, x509keypair_id, values):
        """Update properties of a X509KeyPair.

        :param x509keypair_id: The id or uuid of a X509KeyPair.
        :returns: A X509KeyPair.
        """

    @abc.abstractmethod
    def get_x509keypair_list(self, context, filters=None, limit=None,
                             marker=None, sort_key=None, sort_dir=None):
        """Get matching x509keypairs.

        Return a list of the specified columns for all x509keypairs
        that match the specified filters.

        :param context: The security context
        :param filters: Filters to apply. Defaults to None.

        :param limit: Maximum number of x509keypairs to return.
        :param marker: the last item of the previous page; we return the next
                       result set.
        :param sort_key: Attribute by which results should be sorted.
        :param sort_dir: direction in which results should be sorted.
                         (asc, desc)
        :returns: A list of tuples of the specified columns.
        """

    @abc.abstractmethod
    def destroy_magnum_service(self, magnum_service_id):
        """Destroys a magnum_service record.

        :param magnum_service_id: The id of a magnum_service.
        """

    @abc.abstractmethod
    def update_magnum_service(self, magnum_service_id, values):
        """Update properties of a magnum_service.

        :param magnum_service_id: The id of a magnum_service record.
        """

    @abc.abstractmethod
    def get_magnum_service_by_host_and_binary(self, host, binary):
        """Return a magnum_service record.

        :param host: The host where the binary is located.
        :param binary: The name of the binary.
        :returns: A magnum_service record.
        """

    @abc.abstractmethod
    def create_magnum_service(self, values):
        """Create a new magnum_service record.

        :param values: A dict containing several items used to identify
                       and define the magnum_service record.
        :returns: A magnum_service record.
        """

    @abc.abstractmethod
    def get_magnum_service_list(self, disabled=None, limit=None,
                                marker=None, sort_key=None, sort_dir=None):
        """Get matching magnum_service records.

        Return a list of the specified columns for all magnum_services
        those match the specified filters.

        :param disabled: Filters disbaled services. Defaults to None.
        :param limit: Maximum number of magnum_services to return.
        :param marker: the last item of the previous page; we return the next
                       result set.
        :param sort_key: Attribute by which results should be sorted.
        :param sort_dir: direction in which results should be sorted.
                         (asc, desc)
        :returns: A list of tuples of the specified columns.
        """

    @abc.abstractmethod
    def create_quota(self, values):
        """Create a new Quota record for a resource in a project.

        :param values: A dict containing several items used to identify
                       and track quota for a resource in a project.

                       ::

                        {
                         'id': uuidutils.generate_uuid(),
                         'project_id': 'fake_project',
                         'resource': 'fake_resource',
                         'hard_limit': 'fake_hardlimit',
                        }

        :returns: A quota record.
        """

    @abc.abstractmethod
    def update_quota(self, project_id, values):
        """Update quota record.

        :param project_id: The project id.
        :param values: A dict containing several items used to identify
                       and track quota for a resource in a project.

                       ::

                        {
                         'id': uuidutils.generate_uuid(),
                         'project_id': 'fake_project',
                         'resource': 'fake_resource',
                         'hard_limit': 'fake_hardlimit',
                        }
        :returns:  A quota record.
        """

    @abc.abstractmethod
    def delete_quota(self, project_id, resource):
        """Delete a quota.

        :param project_id: Project id.
        :param resource: resource name.
        """

    @abc.abstractmethod
    def get_quota_by_id(self, context, quota_id):
        """Return a quota.

        :param context: The security context
        :param quota_id: The id of a quota.
        :returns: A quota.
        """

    @abc.abstractmethod
    def get_quota_list(self, context, filters=None, limit=None,
                       marker=None, sort_key=None, sort_dir=None):
        """Get quota list.

        Return a list of the specified columns for all quotas that match the
        specified filters.

        :param context: The security context
        :param filters: Filters to apply. Defaults to None.

        :param limit: Maximum number of clusters to return.
        :param marker: the last item of the previous page; we return the next
                       result set.
        :param sort_key: Attribute by which results should be sorted.
        :param sort_dir: direction in which results should be sorted.
                         (asc, desc)
        :returns: A list of tuples of the specified columns.
        """

    @abc.abstractmethod
    def quota_get_all_by_project_id(self, project_id):
        """Gets Quota record for all the resources in a project.

        :param project_id: Project identifier of the project.

        :returns: Quota record for all resources in a project.
        """

    @abc.abstractmethod
    def get_quota_by_project_id_resource(self, project_id, resource):
        """Gets quota record for the given quota id.

        :param project_id: project id.
        :param resource: resource name.

        :returns: Quota record.
        """

    @abc.abstractmethod
    def get_federation_by_id(self, context, federation_id):
        """Return a federation for a given federation id.

        :param context: The security context
        :param federation_id: The id of a federation
        :returns: A federation
        """

    @abc.abstractmethod
    def get_federation_by_uuid(self, context, federation_uuid):
        """Return a federation for a given federation uuid.

        :param context: The security context
        :param federation_uuid: The uuid of a federation
        :returns: A federation
        """

    @abc.abstractmethod
    def get_federation_by_name(self, context, federation_name):
        """Return a federation for a given federation name.

        :param context: The security context
        :param federation_name: The name of a federation
        :returns: A federation
        """

    @abc.abstractmethod
    def get_federation_list(self, context, limit=None, marker=None,
                            sort_key=None, sort_dir=None, filters=None):
        """Get matching federations.

        Return a list of the specified columns for all federations that
        match the specified filters.

        :param context: The security context
        :param filters: Filters to apply. Defaults to None.

        :param limit: Maximum number of federations to return.
        :param marker: the last item of the previous page; we return the next
                       result set.
        :param sort_key: Attribute by which results should be sorted.
        :param sort_dir: direction in which results should be sorted.
                         (asc, desc)
        :returns: A list of tuples of the specified columns.
        """

    @abc.abstractmethod
    def create_federation(self, values):
        """Create a new federation.

        :param values: A dict containing several items used to identify
                       and track the federation.
                       For example:
                       ::

                    {
                      'uuid': uuidutils.generate_uuid(),
                      'name': 'example',
                      'hostcluster_id': '91c8dd07-14a2-4fd8-b084-915fa53552fd',
                      'properties': 'dns-zone:example.com.'
                    }

        :returns: A federation.
        """

    @abc.abstractmethod
    def destroy_federation(self, federation_id):
        """Destroy a federation.

        This action *will not* destroy the host cluster nor the member
        clusters.
        :param federation_id: The id or uuid of a federation.
        """

    @abc.abstractmethod
    def update_federation(self, federation_id, values):
        """Update properties of a federation.

        :param federation_id: The id or uuid of a federation.
        :param values: A dict containing several items used to identify
                       and track the federation.
                       For example:
                       ::

                    {
                      'uuid': uuidutils.generate_uuid(),
                      'name': 'example',
                      'hostcluster_id': '91c8dd07-14a2-4fd8-b084-915fa53552fd',
                      'properties': 'dns-zone:example.com.'
                    }

        :returns: A federation.
        :raises: FederationNotFound
        """

    @abc.abstractmethod
    def create_nodegroup(self, values):
        """Create a new nodegroup in cluster.

        :param values: A dict containing several items used to identify
                       and track the nodegroup.
                       For example:
                       ::

                    {
                      'uuid': uuidutils.generate_uuid(),
                      'name': 'example',
                      ...
                    }

        :returns: A nodegroup record.
        """

    @abc.abstractmethod
    def destroy_nodegroup(self, cluster_id, nodegroup_id):
        """Destroy a nodegroup.

        :param cluster_id: The uuid of the cluster where the nodegroup
                             belongs to.
        :param nodegroup_id: The id or uuid of the nodegroup
        """

    @abc.abstractmethod
    def update_nodegroup(self, cluster_id, nodegroup_id, values):
        """Update properties of a nodegroup.

        :param cluster_id: The uuid of the cluster where the nodegroup
                             belongs to.
        :param nodegroup_id: The id or uuid of a nodegroup.
        :param values: A dict containing several items used to identify
                       and track the nodegroup.
                       For example:
                       ::

                    {
                      'uuid': uuidutils.generate_uuid(),
                      'name': 'example',
                      ...
                    }

        :returns: A nodegroup record.
        :raises: NodeGroupNotFound
        """

    @abc.abstractmethod
    def get_nodegroup_by_id(self, context, cluster_id, nodegroup_id):
        """Return a nodegroup for a given cluster uuid and nodegroup id.

        :param cluster_id: The uuid of the cluster where the nodegroup
                             belongs to.
        :param nodegroup_id: The id of a nodegroup.

        :returns: A nodegroup record.
        :raises: NodeGroupNotFound
        """

    @abc.abstractmethod
    def get_nodegroup_by_uuid(self, context, cluster_id, nodegroup_uuid):
        """Return a nodegroup for a given cluster uuid and nodegroup uuid.

        :param cluster_id: The uuid of the cluster where the nodegroup
                             belongs to.
        :param nodegroup_uuid: The uuid of a nodegroup.

        :returns: A nodegroup record.
        :raises: NodeGroupNotFound
        """

    @abc.abstractmethod
    def get_nodegroup_by_name(self, context, cluster_id, nodegroup_name):
        """Return a nodegroup for a given cluster uuid and nodegroup name.

        :param cluster_id: The uuid of the cluster where the nodegroup
                             belongs to.
        :param nodegroup_name: The name of a nodegroup.

        :returns: A nodegroup record.
        :raises: NodeGroupNotFound
        """

    @abc.abstractmethod
    def list_cluster_nodegroups(self, context, cluster_id, filters=None,
                                limit=None, marker=None, sort_key=None,
                                sort_dir=None):
        """Get matching nodegroups in a given cluster.

        :param context: The security context
        :param cluster_id: The uuid of the cluster where the nodegroup
                           belongs to.
        :param filters: Filters to apply. Defaults to None.

        :param limit: Maximum number of nodegroups to return.
        :param marker: the last item of the previous page; we return the next
                       result set.
        :param sort_key: Attribute by which results should be sorted.
        :param sort_dir: direction in which results should be sorted.
                         (asc, desc)

        :returns: A list of nodegroup records.
        """

    @abc.abstractmethod
    def get_cluster_nodegroup_count(self, context, cluster_id):
        """Get count of nodegroups in a given cluster.

        :param cluster_id: The uuid of the cluster where the nodegroup
                             belongs to.
        :returns: Count of matching clusters.
        """



# Copyright 2015 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from oslo_utils import encodeutils
import six

from magnum.common import cert_manager
from magnum.common import exception
from magnum.common import short_id
from magnum.common.x509 import operations as x509

import magnum.conf
import os
import shutil
import tempfile

CONDUCTOR_CLIENT_NAME = six.u('Magnum-Conductor')

LOG = logging.getLogger(__name__)
CONF = magnum.conf.CONF


def _generate_ca_cert(issuer_name, context=None):
    """Generate and store ca_cert

    :param issuer_name: CA subject name
    :returns: CA cert uuid and CA cert, CA private key password
    """
    ca_password = short_id.generate_id()
    ca_cert = x509.generate_ca_certificate(issuer_name,
                                           encryption_password=ca_password)
    ca_cert_ref = cert_manager.get_backend().CertManager.store_cert(
        certificate=ca_cert['certificate'],
        private_key=ca_cert['private_key'],
        private_key_passphrase=ca_password,
        name=issuer_name,
        context=context,
    )
    LOG.debug('CA cert is created: %s', ca_cert_ref)
    return ca_cert_ref, ca_cert, ca_password


def _generate_client_cert(issuer_name, ca_cert, ca_password, context=None):
    """Generate and store magnum_client_cert

    :param issuer_name: CA subject name
    :param ca_cert: CA certificate
    :param ca_password: CA private key password
    :returns: Magnum client cert uuid
    """
    client_password = short_id.generate_id()
    # TODO(strigazi): set subject name and organization per driver
    # For RBAC kubernetes cluster we need the client to have:
    # subject_name: admin
    # organization_name system:masters
    # Non kubernetes drivers are not using the certificates fields
    # for authorization
    subject_name = 'admin'
    organization_name = 'system:masters'
    client_cert = x509.generate_client_certificate(
        issuer_name,
        subject_name,
        organization_name,
        ca_cert['private_key'],
        encryption_password=client_password,
        ca_key_password=ca_password,
    )
    magnum_cert_ref = cert_manager.get_backend().CertManager.store_cert(
        certificate=client_cert['certificate'],
        private_key=client_cert['private_key'],
        private_key_passphrase=client_password,
        name=CONDUCTOR_CLIENT_NAME,
        context=context
    )
    LOG.debug('Magnum client cert is created: %s', magnum_cert_ref)
    return magnum_cert_ref


def _get_issuer_name(cluster):
    issuer_name = cluster.name
    # When user create a Cluster without name, the cluster.name is None.
    # We should use cluster.uuid as issuer name.
    if issuer_name is None:
        issuer_name = cluster.uuid
    return issuer_name


def generate_certificates_to_cluster(cluster, context=None):
    """Generate ca_cert and magnum client cert and set to cluster

    :param cluster: The cluster to set CA cert and magnum client cert
    :returns: CA cert uuid and magnum client cert uuid
    """
    try:
        issuer_name = _get_issuer_name(cluster)

        LOG.debug('Start to generate certificates: %s', issuer_name)

        ca_cert_ref, ca_cert, ca_password = _generate_ca_cert(issuer_name,
                                                              context=context)
        magnum_cert_ref = _generate_client_cert(issuer_name,
                                                ca_cert,
                                                ca_password,
                                                context=context)

        cluster.ca_cert_ref = ca_cert_ref
        cluster.magnum_cert_ref = magnum_cert_ref
    except Exception:
        LOG.exception('Failed to generate certificates for Cluster: %s',
                      cluster.uuid)
        raise exception.CertificatesToClusterFailed(cluster_uuid=cluster.uuid)


def get_cluster_ca_certificate(cluster, context=None):
    ca_cert = cert_manager.get_backend().CertManager.get_cert(
        cluster.ca_cert_ref,
        resource_ref=cluster.uuid,
        context=context
    )

    return ca_cert


def get_cluster_magnum_cert(cluster, context=None):
    magnum_cert = cert_manager.get_backend().CertManager.get_cert(
        cluster.magnum_cert_ref,
        resource_ref=cluster.uuid,
        context=context
    )

    return magnum_cert


def create_client_files(cluster, context=None):
    if not os.path.isdir(CONF.cluster.temp_cache_dir):
        LOG.debug("Certificates will not be cached in the filesystem: they "
                  "will be created as tempfiles.")
        ca_cert = get_cluster_ca_certificate(cluster, context)
        magnum_cert = get_cluster_magnum_cert(cluster, context)

        ca_file = tempfile.NamedTemporaryFile(mode="w+")
        ca_file.write(encodeutils.safe_decode(ca_cert.get_certificate()))
        ca_file.flush()

        key_file = tempfile.NamedTemporaryFile(mode="w+")
        key_file.write(encodeutils.safe_decode(
            magnum_cert.get_decrypted_private_key()))
        key_file.flush()

        cert_file = tempfile.NamedTemporaryFile(mode="w+")
        cert_file.write(encodeutils.safe_decode(magnum_cert.get_certificate()))
        cert_file.flush()

    else:
        cached_cert_dir = os.path.join(CONF.cluster.temp_cache_dir,
                                       cluster.uuid)
        cached_ca_file = os.path.join(cached_cert_dir, 'ca.crt')
        cached_key_file = os.path.join(cached_cert_dir, 'client.key')
        cached_cert_file = os.path.join(cached_cert_dir, 'client.crt')

        if not os.path.isdir(cached_cert_dir):
            os.mkdir(cached_cert_dir)

            ca_cert = get_cluster_ca_certificate(cluster, context)
            magnum_cert = get_cluster_magnum_cert(cluster, context)

            ca_file = open(cached_ca_file, "w+")
            ca_file.write(encodeutils.safe_decode(ca_cert.get_certificate()))
            ca_file.flush()

            key_file = open(cached_key_file, "w+")
            key_file.write(encodeutils.safe_decode(
                magnum_cert.get_decrypted_private_key()))
            key_file.flush()

            cert_file = open(cached_cert_file, "w+")
            cert_file.write(
                encodeutils.safe_decode(magnum_cert.get_certificate()))
            cert_file.flush()

            os.chmod(cached_ca_file, 0o600)
            os.chmod(cached_key_file, 0o600)
            os.chmod(cached_cert_file, 0o600)

        else:
            ca_file = open(cached_ca_file, "r")
            key_file = open(cached_key_file, "r")
            cert_file = open(cached_cert_file, "r")

    return ca_file, key_file, cert_file


def sign_node_certificate(cluster, csr, context=None):
    ca_cert = cert_manager.get_backend().CertManager.get_cert(
        cluster.ca_cert_ref,
        resource_ref=cluster.uuid,
        context=context
    )

    node_cert = x509.sign(csr,
                          _get_issuer_name(cluster),
                          ca_cert.get_private_key(),
                          ca_cert.get_private_key_passphrase())
    return node_cert


def delete_certificates_from_cluster(cluster, context=None):
    """Delete ca cert and magnum client cert from cluster

    :param cluster: The cluster which has certs
    """
    for cert_ref in ['ca_cert_ref', 'magnum_cert_ref']:
        try:
            cert_ref = getattr(cluster, cert_ref, None)
            if cert_ref:
                cert_manager.get_backend().CertManager.delete_cert(
                    cert_ref, resource_ref=cluster.uuid, context=context)
        except Exception:
            LOG.warning("Deleting certs is failed for Cluster %s",
                        cluster.uuid)


def delete_client_files(cluster, context=None):
    cached_cert_dir = os.path.join(CONF.cluster.temp_cache_dir,
                                   cluster.uuid)
    try:
        if os.path.isdir(cached_cert_dir):
            shutil.rmtree(cached_cert_dir)
    except Exception:
        LOG.warning("Deleting client files failed for Cluster %s",
                    cluster.uuid)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from oslo_log import log as logging
import pecan
import wsme
from wsme import types as wtypes

from magnum.api import attr_validator
from magnum.api.controllers import base
from magnum.api.controllers import link
from magnum.api.controllers.v1 import collection
from magnum.api.controllers.v1 import types
from magnum.api import expose
from magnum.api import utils as api_utils
from magnum.api import validation
from magnum.common import exception
from magnum.common import name_generator
from magnum.common import policy
import magnum.conf
from magnum import objects
from magnum.objects import fields

LOG = logging.getLogger(__name__)
CONF = magnum.conf.CONF


class FederationID(wtypes.Base):
    """API representation of a federation ID

    This class enforces type checking and value constraints, and converts
    between the internal object model and the API representation of a
    federation ID.
    """
    uuid = types.uuid

    def __init__(self, uuid):
        self.uuid = uuid


class Federation(base.APIBase):
    """API representation of a federation.

    This class enforces type checking and value constraints, and converts
    between the internal object model and the API representation of a
    Federation.
    """

    # Unique UUID for this federation.
    uuid = types.uuid

    # Name of this federation, max length is limited to 242 because heat stack
    # requires max length limit to 255, and Magnum amend a uuid length.
    name = wtypes.StringType(min_length=1, max_length=242,
                             pattern='^[a-zA-Z][a-zA-Z0-9_.-]*$')

    # UUID of the hostcluster of the federation, i.e. the cluster that
    # hosts the COE Federated API.
    hostcluster_id = wsme.wsattr(wtypes.text)

    # List of UUIDs of all the member clusters of the federation.
    member_ids = wsme.wsattr([wtypes.text])

    # Status of the federation.
    status = wtypes.Enum(wtypes.text, *fields.FederationStatus.ALL)

    # Status reason of the federation.
    status_reason = wtypes.text

    # Set of federation metadata (COE-specific in some cases).
    properties = wtypes.DictType(wtypes.text, wtypes.text)

    # A list containing a self link and associated federations links
    links = wsme.wsattr([link.Link], readonly=True)

    def __init__(self, **kwargs):
        super(Federation, self).__init__()
        self.fields = []
        for field in objects.Federation.fields:
            # Skip fields we do not expose.
            if not hasattr(self, field):
                continue
            self.fields.append(field)
            setattr(self, field, kwargs.get(field, wtypes.Unset))

    @staticmethod
    def _convert_with_links(federation, url, expand=True):
        if not expand:
            federation.unset_fields_except(['uuid', 'name', 'hostcluster_id',
                                            'member_ids', 'status',
                                            'properties'])

        federation.links = [link.Link.make_link('self', url, 'federations',
                                                federation.uuid),
                            link.Link.make_link('bookmark', url, 'federations',
                                                federation.uuid,
                                                bookmark=True)]
        return federation

    @classmethod
    def convert_with_links(cls, rpc_federation, expand=True):
        federation = Federation(**rpc_federation.as_dict())
        return cls._convert_with_links(federation, pecan.request.host_url,
                                       expand)

    @classmethod
    def sample(cls, expand=True):
        sample = cls(uuid='4221a353-8368-475f-b7de-3429d3f724b3',
                     name='example',
                     hostcluster_id='49dc23f5-ffc9-40c3-9d34-7be7f9e34d63',
                     member_ids=['49dc23f5-ffc9-40c3-9d34-7be7f9e34d63',
                                 'f2439bcf-02a2-4278-9d8a-f07a2042230a',
                                 'e549e0a5-3d3c-406f-bd7c-0e0182fb211c'],
                     properties={'dns-zone': 'example.com.'},
                     status=fields.FederationStatus.CREATE_COMPLETE,
                     status_reason="CREATE completed successfully")
        return cls._convert_with_links(sample, 'http://localhost:9511', expand)


class FederationPatchType(types.JsonPatchType):
    _api_base = Federation

    @staticmethod
    def internal_attrs():
        """"Returns a list of internal attributes.

        Internal attributes can't be added, replaced or removed.
        """
        internal_attrs = []
        return types.JsonPatchType.internal_attrs() + internal_attrs


class FederationCollection(collection.Collection):
    """API representation of a collection of federations."""

    # A list containing federation objects.
    federations = [Federation]

    def __init__(self, **kwargs):
        self._type = 'federations'

    @staticmethod
    def convert_with_links(rpc_federation, limit, url=None, expand=False,
                           **kwargs):
        collection = FederationCollection()
        collection.federations = [Federation.convert_with_links(p, expand)
                                  for p in rpc_federation]
        collection.next = collection.get_next(limit, url=url, **kwargs)
        return collection

    @classmethod
    def sample(cls):
        sample = cls()
        sample.federations = [Federation.sample(expand=False)]
        return sample


class FederationsController(base.Controller):
    """REST controller for federations."""

    def __init__(self):
        super(FederationsController, self).__init__()

    _custom_actions = {
        'detail': ['GET'],
    }

    def _generate_name_for_federation(self, context):
        """Generate a random name like: phi-17-federation."""
        name_gen = name_generator.NameGenerator()
        name = name_gen.generate()
        return name + '-federation'

    def _get_federation_collection(self, marker, limit,
                                   sort_key, sort_dir, expand=False,
                                   resource_url=None):
        limit = api_utils.validate_limit(limit)
        sort_dir = api_utils.validate_sort_dir(sort_dir)

        marker_obj = None
        if marker:
            marker_obj = objects.Federation.get_by_uuid(pecan.request.context,
                                                        marker)

        federations = objects.Federation.list(pecan.request.context, limit,
                                              marker_obj, sort_key=sort_key,
                                              sort_dir=sort_dir)

        return FederationCollection.convert_with_links(federations, limit,
                                                       url=resource_url,
                                                       expand=expand,
                                                       sort_key=sort_key,
                                                       sort_dir=sort_dir)

    @expose.expose(FederationCollection, types.uuid, int, wtypes.text,
                   wtypes.text)
    def get_all(self, marker=None, limit=None, sort_key='id',
                sort_dir='asc'):
        """Retrieve a list of federations.

        :param marker: pagination marker for large data sets.
        :param limit: maximum number of resources to return in a single result.
        :param sort_key: column to sort results by. Default: id.
        :param sort_dir: direction to sort. "asc" or "desc". Default: asc.
        """
        context = pecan.request.context
        policy.enforce(context, 'federation:get_all',
                       action='federation:get_all')
        return self._get_federation_collection(marker, limit, sort_key,
                                               sort_dir)

    @expose.expose(FederationCollection, types.uuid, int, wtypes.text,
                   wtypes.text)
    def detail(self, marker=None, limit=None, sort_key='id',
               sort_dir='asc'):
        """Retrieve a list of federation with detail.

        :param marker: pagination marker for large data sets.
        :param limit: maximum number of resources to return in a single result.
        :param sort_key: column to sort results by. Default: id.
        :param sort_dir: direction to sort. "asc" or "desc". Default: asc.
        """
        context = pecan.request.context
        policy.enforce(context, 'federation:detail',
                       action='federation:detail')

        # NOTE(lucasagomes): /detail should only work against collections
        parent = pecan.request.path.split('/')[:-1][-1]
        if parent != "federations":
            raise exception.HTTPNotFound

        expand = True
        resource_url = '/'.join(['federations', 'detail'])
        return self._get_federation_collection(marker, limit,
                                               sort_key, sort_dir, expand,
                                               resource_url)

    @expose.expose(Federation, types.uuid_or_name)
    def get_one(self, federation_ident):
        """Retrieve information about a given Federation.

        :param federation_ident: UUID or logical name of the Federation.
        """
        context = pecan.request.context
        federation = api_utils.get_resource('Federation', federation_ident)
        policy.enforce(context, 'federation:get', federation.as_dict(),
                       action='federation:get')

        federation = Federation.convert_with_links(federation)

        return federation

    @expose.expose(FederationID, body=Federation, status_code=202)
    def post(self, federation):
        """Create a new federation.

        :param federation: a federation within the request body.
        """
        context = pecan.request.context
        policy.enforce(context, 'federation:create',
                       action='federation:create')

        federation_dict = federation.as_dict()

        # Validate `hostcluster_id`
        hostcluster_id = federation_dict.get('hostcluster_id')
        attr_validator.validate_federation_hostcluster(hostcluster_id)

        # Validate `properties` dict.
        properties_dict = federation_dict.get('properties')
        attr_validator.validate_federation_properties(properties_dict)

        federation_dict['project_id'] = context.project_id

        # If no name is specified, generate a random human-readable name
        name = (federation_dict.get('name') or
                self._generate_name_for_federation(context))
        federation_dict['name'] = name

        new_federation = objects.Federation(context, **federation_dict)
        new_federation.uuid = uuid.uuid4()

        # TODO(clenimar): remove hard-coded `create_timeout`.
        pecan.request.rpcapi.federation_create_async(new_federation,
                                                     create_timeout=15)

        return FederationID(new_federation.uuid)

    @expose.expose(FederationID, types.uuid_or_name, types.boolean,
                   body=[FederationPatchType], status_code=202)
    def patch(self, federation_ident, rollback=False, patch=None):
        """Update an existing Federation.

        Please note that the join/unjoin operation is performed by patching
        `member_ids`.

        :param federation_ident: UUID or logical name of a federation.
        :param rollback: whether to rollback federation on update failure.
        :param patch: a json PATCH document to apply to this federation.
        """
        federation = self._patch(federation_ident, patch)
        pecan.request.rpcapi.federation_update_async(federation, rollback)
        return FederationID(federation.uuid)

    def _patch(self, federation_ident, patch):
        context = pecan.request.context
        federation = api_utils.get_resource('Federation', federation_ident)
        policy.enforce(context, 'federation:update', federation.as_dict(),
                       action='federation:update')

        # NOTE(clenimar): Magnum does not allow one to append items to existing
        # fields through an `add` operation using HTTP PATCH (please check
        # `magnum.api.utils.apply_jsonpatch`). In order to perform the join
        # and unjoin operations, intercept the original JSON PATCH document
        # and change the operation from either `add` or `remove` to `replace`.
        patch_path = patch[0].get('path')
        patch_value = patch[0].get('value')
        patch_op = patch[0].get('op')

        if patch_path == '/member_ids':
            if patch_op == 'add' and patch_value is not None:
                patch = self._join_wrapper(federation_ident, patch)
            elif patch_op == 'remove' and patch_value is not None:
                patch = self._unjoin_wrapper(federation_ident, patch)

        try:
            federation_dict = federation.as_dict()
            new_federation = Federation(
                **api_utils.apply_jsonpatch(federation_dict, patch))
        except api_utils.JSONPATCH_EXCEPTIONS as e:
            raise exception.PatchError(patch=patch, reason=e)

        # Retrieve only what changed after the patch.
        delta = self._update_changed_fields(federation, new_federation)
        validation.validate_federation_properties(delta)

        return federation

    def _update_changed_fields(self, federation, new_federation):
        """Update only the patches that were modified and return the diff."""
        for field in objects.Federation.fields:
            try:
                patch_val = getattr(new_federation, field)
            except AttributeError:
                # Ignore fields that aren't exposed in the API
                continue
            if patch_val == wtypes.Unset:
                patch_val = None
            if federation[field] != patch_val:
                federation[field] = patch_val

        return federation.obj_what_changed()

    def _join_wrapper(self, federation_ident, patch):
        """Intercept PATCH JSON documents for join operations.

        Take a PATCH JSON document with `add` operation::
            {
                'op': 'add',
                'value': 'new_member_id',
                'path': '/member_ids'
            }
        and transform it into a document with `replace` operation::
            {
                'op': 'replace',
                'value': ['current_member_id1', ..., 'new_member_id'],
                'path': '/member_ids'
            }
        """
        federation = api_utils.get_resource('Federation', federation_ident)
        new_member_uuid = patch[0]['value']

        # Check if the cluster exists
        c = objects.Cluster.get_by_uuid(pecan.request.context, new_member_uuid)

        # Check if the cluster is already a member of the federation
        if new_member_uuid not in federation.member_ids and c is not None:
            # Retrieve all current members
            members = federation.member_ids
            # Add the new member
            members.append(c.uuid)
        else:
            kw = {'uuid': new_member_uuid, 'federation_name': federation.name}
            raise exception.MemberAlreadyExists(**kw)

        # Set `value` to the updated member list. Change `op` to `replace`
        patch[0]['value'] = members
        patch[0]['op'] = 'replace'

        return patch

    def _unjoin_wrapper(self, federation_ident, patch):
        """Intercept PATCH JSON documents for unjoin operations.

        Take a PATCH JSON document with `remove` operation::
            {
                'op': 'remove',
                'value': 'former_member_id',
                'path': '/member_ids'
            }
        and transform it into a document with `replace` operation::
            {
                'op': 'replace',
                'value': ['current_member_id1', ..., 'current_member_idn'],
                'path': '/member_ids'
            }
        """
        federation = api_utils.get_resource('Federation', federation_ident)
        cluster_uuid = patch[0]['value']

        # Check if the cluster exists
        c = objects.Cluster.get_by_uuid(pecan.request.context, cluster_uuid)

        # Check if the cluster is a member cluster and if it exists
        if cluster_uuid in federation.member_ids and c is not None:
            # Retrieve all current members
            members = federation.member_ids
            # Unjoin the member
            members.remove(cluster_uuid)
        else:
            raise exception.HTTPNotFound("Cluster %s is not a member of the "
                                         "federation %s." % (cluster_uuid,
                                                             federation.name))

        # Set `value` to the updated member list. Change `op` to `replace`
        patch[0]['value'] = members
        patch[0]['op'] = 'replace'

        return patch

    @expose.expose(None, types.uuid_or_name, status_code=204)
    def delete(self, federation_ident):
        """Delete a federation.

        :param federation_ident: UUID of federation or logical name
                                 of the federation.
        """
        context = pecan.request.context
        federation = api_utils.get_resource('Federation', federation_ident)
        policy.enforce(context, 'federation:delete', federation.as_dict(),
                       action='federation:delete')

        pecan.request.rpcapi.federation_delete_async(federation.uuid)



# Copyright 2012 New Dream Network, LLC (DreamHost)
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pecan
from pecan import rest
from wsme import types as wtypes

from magnum.api.controllers import base
from magnum.api.controllers import link
from magnum.api.controllers import v1
from magnum.api.controllers import versions
from magnum.api import expose


class Version(base.APIBase):
    """An API version representation."""

    id = wtypes.text
    """The ID of the version, also acts as the release number"""

    links = [link.Link]
    """A Link that point to a specific version of the API"""

    status = wtypes.text
    """The current status of the version: CURRENT, SUPPORTED, UNSUPPORTED"""

    max_version = wtypes.text
    """The max microversion supported by this version"""

    min_version = wtypes.text
    """The min microversion supported by this version"""

    @staticmethod
    def convert(id, status, max, min):
        version = Version()
        version.id = id
        version.links = [link.Link.make_link('self', pecan.request.host_url,
                                             id, '', bookmark=True)]
        version.status = status
        version.max_version = max
        version.min_version = min
        return version


class Root(base.APIBase):

    name = wtypes.text
    """The name of the API"""

    description = wtypes.text
    """Some information about this API"""

    versions = [Version]
    """Links to all the versions available in this API"""

    @staticmethod
    def convert():
        root = Root()
        root.name = "OpenStack Magnum API"
        root.description = ("Magnum is an OpenStack project which aims to "
                            "provide container cluster management.")
        root.versions = [Version.convert('v1', "CURRENT",
                                         versions.CURRENT_MAX_VER,
                                         versions.BASE_VER)]
        return root


class RootController(rest.RestController):

    _versions = ['v1']
    """All supported API versions"""

    _default_version = 'v1'
    """The default API version"""

    v1 = v1.Controller()

    @expose.expose(Root)
    def get(self):
        # NOTE: The reason why convert() it's being called for every
        #       request is because we need to get the host url from
        #       the request object to make the links.
        return Root.convert()

    @pecan.expose()
    def _route(self, args):
        """Overrides the default routing behavior.

        It redirects the request to the default version of the magnum API
        if the version number is not specified in the url.
        """

        if args[0] and args[0] not in self._versions:
            args = [self._default_version] + args
        return super(RootController, self)._route(args)
